
#include <trace.h>
#include "Headers\sAPI_Error.h"
#include "Headers\sAPI_Comm.h"
#include "Headers\sAPI_Helper.h"


const struct COMMOPTIONS sAPI_Comm::CommOption1 = {CBR_2400,DTR_CONTROL_ENABLE,false,ODDPARITY,1500,40,40,false,mmMEMORYMODEL_1};
const struct COMMOPTIONS sAPI_Comm::CommOption2 = {CBR_9600,DTR_CONTROL_DISABLE,true,NOPARITY,1100,0x100,0x100,true,mmMEMORYMODEL_2};
const struct COMMOPTIONS sAPI_Comm::CommOption3 = {CBR_9600,DTR_CONTROL_ENABLE,false,NOPARITY,3400,0x100,0x100,false,mmMEMORYMODEL_3};

const int sAPI_Comm::Command_Read = 0x05;
const int sAPI_Comm::Command_PreWrite = 0x07;
const int sAPI_Comm::Command_Write = 0x06;
const int sAPI_Comm::Command_FirstDive = 0x08;
const int sAPI_Comm::Command_NextDive = 0x09;
const int sAPI_Comm::Command_Command = 0xA5;
const int sAPI_Comm::Command_Unknown = 0x0A;
const int sAPI_Comm::Command_Reset = 0x0B;

const wchar_t * const ComStateError = L"Error Setting Comm Port";

sAPI_Comm::sAPI_Comm(const wchar_t *port,struct COMMOPTIONS CO) : fPortOpen(false),fCommDelay(6),
				fCommPort(INVALID_HANDLE_VALUE),fRead(CO.Read),fMemoryModel(CO.Memory),
				fEventMask(EV_TXEMPTY),fState(_READ)
 {
		   // set up comport
	CommandBuffer = new unsigned char [CO.CommandBufferSize];
	DataBuffer = new unsigned char [CO.DataBufferSize];
	   
     if(port == 0) 
     {
   #ifndef _DEBUG
       	Trace(TraceError,"sAPI_Comm::Create() Unable To Open Port <No Port Specified>");
		throw sAPI_CriticalError(NO_PORT_SPECIFED);
   #else // ADDED for 'file' reader support ONLY
		fPortOpen =true;
		return;
   #endif
	 }
	   fCommPort = ::CreateFileW(port,
					GENERIC_READ | GENERIC_WRITE,
					0,    // must be opened with exclusive-access
					NULL, // no security attributes
					OPEN_EXISTING, // must use OPEN_EXISTING
					0,    // not overlapped I/O
					NULL  // hTemplate must be NULL for comm devices
					);
		 if(fCommPort == INVALID_HANDLE_VALUE)
		  {
			  Trace(TraceError,"sAPI_Comm::Create() Unable To Open Port <Port Busy or Not Present>");
			  throw sAPI_CommError(BAD_OPEN,L"Comm Port Not Found");
		  }

			// get Old DCB and Old Timeouts
		 OldCommControlBlock.DCBlength = sizeof (DCB);
		 if( (0 == GetCommState(fCommPort, &OldCommControlBlock))
			 || (0 == GetCommTimeouts( fCommPort, &OldCommTimeOutControl )) )
		  {
			  CloseHandle(fCommPort);
			  Trace(TraceError,"sAPI_Comm::Create() Can't Get Comm state or timeouts");
			  throw sAPI_CommError(BAD_OPEN,L"Invalid Port Specified");
		  }

		Trace(TraceDetail,"sAPI_Comm::Create() Port Opened (%d)",fCommPort);

		 CommControlBlock.DCBlength = sizeof (DCB);
		 if( 0== ::GetCommState(fCommPort,&CommControlBlock))
		  {
			  CloseHandle(fCommPort);
			  Trace(TraceError,"sAPI_Comm::Create() Can't Fill DCB with old Comm State");
			  throw sAPI_CriticalError(ComStateError);
		  }

		 //CommControlBlock.BaudRate = ((fMemoryModel == mmMEMORYMODEL_2)? CBR_9600 : CBR_2400);
		 CommControlBlock.fBinary = true;               // Binary mode; no EOF check
		 CommControlBlock.fParity = false;
         CommControlBlock.fOutxCtsFlow = false;
		 CommControlBlock.fOutxDsrFlow = false;
		 //CommControlBlock.fDtrControl = ((fMemoryModel == mmMEMORYMODEL_2)? DTR_CONTROL_DISABLE : DTR_CONTROL_ENABLE);
		 CommControlBlock.fDsrSensitivity = false;
		 CommControlBlock.fTXContinueOnXoff = false;      // XOFF continues Tx
		 CommControlBlock.fOutX = false;                // No XON/XOFF out flow control
         CommControlBlock.fInX = false;                 // No XON/XOFF in flow control 
         CommControlBlock.fErrorChar = false;           // Disable error replacement 
         CommControlBlock.fNull = false;                // Disable null stripping 
		 CommControlBlock.fRtsControl = RTS_CONTROL_DISABLE;
		 //CommControlBlock.fAbortOnError = ((fMemoryModel == mmMEMORYMODEL_2)?  true : false);  // Do not abort reads/writes on
         CommControlBlock.XonLim = 2048;
         CommControlBlock.XoffLim = 512;
		 CommControlBlock.ByteSize = 8;
		 //CommControlBlock.Parity = ((fMemoryModel == mmMEMORYMODEL_2)?  NOPARITY : ODDPARITY);
         CommControlBlock.StopBits = ONESTOPBIT;
		 CommControlBlock.XonChar = 0x11;
		 CommControlBlock.XoffChar = 0x13;
		 CommControlBlock.ErrorChar = 0x00;
		 CommControlBlock.EofChar = 0x00;
		 CommControlBlock.EvtChar = 0x00;


		 CommControlBlock.BaudRate = CO.BaudRate;
		 CommControlBlock.fDtrControl = CO.fDtrControl;
		 CommControlBlock.fAbortOnError = CO.fAbortOnError;
		 CommControlBlock.Parity = CO.Parity;

		 if(0==::SetCommState(fCommPort,&CommControlBlock))
		  {
              CloseHandle(fCommPort);
			  Trace(TraceError,"sAPI_Comm::Create() Can't Set Comm state");
			  throw sAPI_CriticalError(ComStateError);
		  }
		 if(0==::GetCommTimeouts( fCommPort, &CommTimeOutControl ))
		  {
			  CloseHandle(fCommPort);
			  Trace(TraceError,"sAPI_Comm::Create() Can't Get Timeouts");
			  throw sAPI_CriticalError(ComStateError);
		  }
		 CommTimeOutControl.ReadIntervalTimeout =  0;
		 CommTimeOutControl.ReadTotalTimeoutMultiplier = 0;
		 CommTimeOutControl.ReadTotalTimeoutConstant = CO.ReadTotalTimeoutConstant;
		 CommTimeOutControl.WriteTotalTimeoutMultiplier = 0;
		 CommTimeOutControl.WriteTotalTimeoutConstant = 1000;

		 if(0==::SetCommTimeouts( fCommPort, &CommTimeOutControl ))
		  {
			  ::SetCommState(fCommPort,&OldCommControlBlock);
			  Trace(TraceError,"sAPI_Comm::Create() Can't Set New Timeouts");
			  CloseHandle(fCommPort);
			  throw sAPI_CriticalError(ComStateError);
		  }

		 ::GetCommMask(fCommPort,&fOldEventMask);
		 fEventMask = EV_TXEMPTY;
		 if(0==::SetCommMask(fCommPort, EV_TXEMPTY))
          {
              ::SetCommState(fCommPort,&OldCommControlBlock);
			  Trace(TraceError,"sAPI_Comm::Create() Can't Set CommMask");
			  CloseHandle(fCommPort);
			  throw sAPI_CriticalError(ComStateError);
		  }

		 if(0==::SetupComm(fCommPort,32640,1280))
          {
              ::SetCommState(fCommPort,&OldCommControlBlock);
			  Trace(TraceError,"sAPI_Comm::Create() Invalid Setup");
			  CloseHandle(fCommPort);
			  throw sAPI_CriticalError(ComStateError);
		  }
		 fPortOpen =true;
		 SetDTR(true);
 }

sAPI_Comm::~sAPI_Comm()
 {
	if(CommandBuffer) delete [] CommandBuffer;
 	if(DataBuffer) delete [] DataBuffer;
	Close();
 }

void sAPI_Comm::Close(void) throw()
 {
	if((fPortOpen==false) || (fCommPort == INVALID_HANDLE_VALUE)) return;
	::SetCommMask(fCommPort,fOldEventMask);
    ::SetCommState(fCommPort,&OldCommControlBlock);
	::SetCommTimeouts( fCommPort, &OldCommTimeOutControl );
	::CloseHandle(fCommPort);
    fPortOpen = false;
 }

int sAPI_Comm::delay(int val) throw()
 {
    if(val == 0 ) return fCommDelay;
    if(val < 1 ) val = 1;
    else if(val > 12) val = 12;
    return (fCommDelay = val);
 }


void sAPI_Comm::SetRTS( bool Val ) throw(sAPI_UnknownError)
 {
	DWORD z;
	if(Val) z = SETRTS;
	else z = CLRRTS;
	if(0==::EscapeCommFunction(fCommPort,z)) throw sAPI_UnknownError(L"SetRTS");
 }

void sAPI_Comm::SetDTR(bool Val ) throw(sAPI_UnknownError)
 {
	DWORD z;
	if(Val) z = SETDTR;
	else z = CLRDTR;
	if(0==::EscapeCommFunction(fCommPort,z))  throw sAPI_UnknownError(L"SetDTR");
 }

unsigned char sAPI_Comm::getchecksum( const unsigned char *package, int len ) throw(sAPI_CriticalError)
 {
	unsigned char checksum = 0x00;
	if(len < 1) throw sAPI_CriticalError(L"Attempt to checksum Null buffer");
	for( ; len > 0; --len) checksum ^= *package++;
	return checksum;
 }

int sAPI_Comm::checkpackagechecksum(unsigned char *dataptr,int Count) throw()
 {
	unsigned char c;
	try{
		c = getchecksum(dataptr,Count); // builds a check sum for command + Count + data[count]
	 }
	catch(...) { return -1; } // change this if needed to rethrow
	return ( c - dataptr[Count]);  // return ZERO if Equal - nonzero if bad CRC or sAPI_Error
 }

