#ifndef sAPI_CommH
#define sAPI_CommH

#include <windows.h>
#include <iostream>
#include <fstream>
#include <boost/utility.hpp>

#include "Headers/sAPI_Error.h"
#include "Headers/sAPI_Helper.h"

#define _WRITE 1
#define _READ 0

struct COMMOPTIONS
 {
    DWORD BaudRate;
    DWORD fDtrControl;
    DWORD fAbortOnError;
    BYTE  Parity;
	DWORD ReadTotalTimeoutConstant;
	int CommandBufferSize;
	int DataBufferSize;
	bool Read;
    enum MemoryModel Memory;
 };


class sAPI_Comm : boost::noncopyable
 {
    private:
		size_t fTry;
		HANDLE fCommPort;
        bool fPortOpen;
        int fCommDelay;
		bool fRead;
		bool fState;
		DWORD fOldEventMask;
        DCB CommControlBlock;
        DCB OldCommControlBlock;
        COMMTIMEOUTS CommTimeOutControl;
        COMMTIMEOUTS OldCommTimeOutControl;
		enum MemoryModel fMemoryModel;

	protected:
    	size_t Tries(void) const throw() { return fTry; }
		virtual void SetCommState(bool state) throw() =0;
		virtual void Talk(int b) throw(sAPI_Error) =0;
		virtual void Listen(int b) throw(sAPI_Error) =0;
		void SetRTS( bool Val ) throw (sAPI_UnknownError);
		void SetDTR(bool Val ) throw (sAPI_UnknownError);
		COMSTAT  ComStat;
		DWORD ComErrorMask; //CE_BREAK | CE_FRAME | CE_IOE | CE_RXPARITY | CE_OVERRUN | CE_RXOVER | CE_TXFULL;
		DWORD fEventMask;
 		unsigned char getchecksum( const unsigned char *package, int len ) throw(sAPI_CriticalError);
		int checkpackagechecksum(unsigned char *dataptr,int Count) throw();
		
	public:
		sAPI_Comm(const wchar_t *port,struct COMMOPTIONS CO,size_t tries =3);
		virtual ~sAPI_Comm();

		unsigned char *CommandBuffer;
		unsigned char *DataBuffer;
		inline HANDLE commport() const throw() { return fCommPort; }
		inline bool commopen(void)const throw() { return fPortOpen; }
		int delay(int d) throw();
		void setstate(bool s) { fState = s; }
		bool getstate(void) const throw() { return fState; }
		void inline Purge(void) const throw() { ::PurgeComm(fCommPort,PURGE_RXCLEAR | PURGE_TXCLEAR); }
		virtual void Close(void) throw();
        virtual void Communicate(int s, int r) throw(sAPI_Error) =0;
		const static int Command_Read;
		const static int Command_PreWrite;
		const static int Command_Write;
		const static int Command_FirstDive;
		const static int Command_NextDive;
		const static int Command_Command;
		const static int Command_Unknown;
		const static int Command_Reset;
		const static struct COMMOPTIONS CommOption1;
		const static struct COMMOPTIONS CommOption2;
		const static struct COMMOPTIONS CommOption3;
 };

class sAPI_CommModel1 : public sAPI_Comm
 {
    private:
        bool fState;
		void SetCommState(bool state) throw();

	protected:
		int GetDataPackage(unsigned char *Buffer,int flag) throw(sAPI_CommError);;
		virtual void Talk(int b) throw(sAPI_Error);
		virtual void Listen(int b) throw(sAPI_Error);

	public:
		sAPI_CommModel1(const wchar_t *port, int d);
        virtual ~sAPI_CommModel1();
        void Communicate(int s, int r) throw(sAPI_Error) { Talk(s); Listen(r); }
 };

class sAPI_CommModel2 : public sAPI_Comm
 {
	private:
		size_t fTry;
		bool fState;
		void SetCommState(bool state) throw();

	protected:
		void GetDeviceHeader(unsigned char *data);
		virtual void Talk(int b) throw(sAPI_Error);
		virtual void Listen(int b) throw(sAPI_Error);

	public:
		sAPI_CommModel2(const wchar_t *port,size_t tries =3);
		virtual ~sAPI_CommModel2();
        void Communicate(int s, int r) throw(sAPI_Error);
 };

class sAPI_CommModel3 : public sAPI_Comm
 {
	private:
		size_t fTry;
		bool fState;
		void SetCommState(bool state) throw();

	protected:
		void GetDeviceHeader(unsigned char *data);
		virtual void Talk(int b) throw(sAPI_Error);
		virtual void Listen(int b) throw(sAPI_Error);

	public:
		sAPI_CommModel3(const wchar_t *port,size_t tries =3);
		~sAPI_CommModel3();
        void Communicate(int s, int r) throw(sAPI_Error);
 };

class sAPI_CommModel2F : public sAPI_Comm
 {
    private:
        bool fState;
		void SetCommState(bool state) throw();
		virtual void Talk(int b)throw(sAPI_Error);
		virtual void Listen(int b)throw(sAPI_Error);
		std::ifstream infile;
		bool fCommOpen;
		unsigned char *file;
		unsigned char AddressHigh;
		unsigned char AddressLow;
		unsigned char AddressSize;
		const wchar_t * fPort;

    protected:

    public:
		sAPI_CommModel2F(const wchar_t *port,size_t tries =3);
		~sAPI_CommModel2F();
		void __stdcall Open(int z) throw(sAPI_Error);
        void Communicate(int s, int r) throw(sAPI_Error) { Talk(s); Listen(r); }
 };

 #endif
