#ifndef GeneraterT2H
	#define GeneraterT2H

#include <stdio.h>
#include <boost\shared_ptr.hpp>
#include "Headers\TGenerater.h"
#include "Headers\sAPI_Stack.h"


#define PRESSURE 1
#define TEMPERATURE 2
#define TIME	4
#define HEADING 8
#define PUSHEDEVENT 64

#define EVENT_STACK_SIZE 200


extern char *rgbm_str[];

struct mdlookup
 {
	 char *Name;
	 unsigned char ID;
 };
 
struct  T2NodeStruct
 {
 	DCDML_Node DCDMLNode;
	DCDML_Node deviceNode;
 	DCDML_Node settingsNode;
 	DCDML_Node historyNode;
 	DCDML_Node profileNode;
 };


struct MixType
{
    std::string name;
	unsigned mixO2;
	int startpress;
	int endpress;
 	MixType(const char* const n ="",unsigned o =0,int s=0,int e=0) : name(n),mixO2(o),startpress(s),endpress(e) {}
	~MixType() {}
};

typedef std::list<boost::shared_ptr<struct MixType> > MIXLIST;

class OutputGeneraterT2Base : public OutputGenerater
 {
    protected:
		sAPI_Stack eventstack;
		DIVES *fDives;
		pDEVICE_DATA fDeviceInfo;
        int fOutputUnits;
        UnitsConverter *Units;
       	TVersionInformation version;
        MIXLIST MixList;
        static const char * const event[];
        static const std::string DcdmlBase;
 		static const char * const devicestrings[];
        static const char * const historystrings[];
        //static const char * const settingsstrings[];
		static const char * const units_str;
        static const char * const value_str;
		static const char * const time_str;
		static const char * const id_str;
		static const char * const rgbm_str[];

        char stringbuffer[200];

		const char * const getversioninfo(void) throw();
		const char * const UC2str(unsigned char v) throw();
		const char * const US2str(unsigned short v) throw();
		const char * const Double2depthstr(double v, unsigned p) throw();
		const char * const Double2pressstr(double v, unsigned p) throw();
		const char * const getmodelstr(void) const throw();
		const char * const getlocaltime(void) throw();
        const char * const getpersonalsetting(unsigned s) throw();
        const char * const getaltitudesetting(unsigned s) throw();
		const char * const makedivetime(int t) throw();
		const char * const makemaxdepth(int d) throw();
        const char * const getdivemode(void) throw();
		void settimestamp(DCDML_Node Node, DIVE d) throw();
        const char * const getmode(DIVE d) throw();
        const char * const addgastolist(unsigned O2,int sp,int ep) throw();
		const char * const addgastolist(const char *name,unsigned O2,int sp,int ep) throw();
        void addgasmix(struct MixType &M, DCDML_Node Node) throw();
 		void basicadddepth(DCDML_Node Node,int depth, unsigned options,int pressure,int temp,int time) throw();
        void dogasswitch(DCDML_Node Node,unsigned O2,int time) throw();
        void builduserevent(DCDML_Node N,TEvent*E) throw();
		void addgasswitch(DCDML_Node Node,const char *name,int time=0,bool option=0) throw();

        void basic_generate(DCDMLDocument& Doc,DCDML_Node DCDMLNode)  throw(sAPI_CriticalError);
        void generate_generater(DCDML_Node DCDMLNode) throw();

	public:
		OutputGeneraterT2Base(pDEVICE_DATA p,DIVES *d,int m,int u)
			: OutputGenerater(m), fDeviceInfo(p),fDives(d),
            Units(0), fOutputUnits(u), eventstack(200)

         { }
		virtual ~OutputGeneraterT2Base()
         {
         	if(Units) delete Units;
         	MixList.clear();
         }
		virtual void Generate(DCDMLDocument& Doc)=0;
 };

 class OutputGeneraterT2D1 : public OutputGeneraterT2Base
 {
	private:
        char *getpersonalinfo(void);
        void dodevice( DCDML_Node deviceNode);
		char *buildserialnumber(void);
		void dohistory(DCDML_Node historyNode);
        void dosettings( DCDML_Node settingsNode);
        void builddive(DCDML_Node ParentNode,DIVES &D);
        void doprofile(DCDML_Node Node,DIVES *Data);
        void dosamples(DCDML_Node ParentNode, DIVE d) throw(sAPI_Error);
        void addevent(DCDML_Node Node) throw();
		void adddepth(DCDML_Node Node,int depth,unsigned options=0,int pressure=0,int temp=0,int time=0) throw();
        const char * const doevent(unsigned char c) throw();
        const char * const getrgbmsetting(unsigned s) throw();
	public:
		OutputGeneraterT2D1(pDEVICE_DATA p,DIVES *d, int m,int u) : OutputGeneraterT2Base(p,d,m,u)
         { }
		~OutputGeneraterT2D1(){}
		virtual void Generate(DCDMLDocument& Doc);
 };

 class OutputGeneraterT2_D23Root : public OutputGeneraterT2Base
 {
	protected:

	    void dohistory(DCDML_Node historyNode) throw();
        void dodevice( DCDML_Node deviceNode) throw();
		char *buildserialnumber(void) throw();
      //  virtual void dosettings( DCDML_Node settingsNode) throw()=0;
        void doprofile(DCDML_Node Node,DIVES *Data) throw();
        void dotissuedata(DCDML_Node DiveNode, DIVE d) throw();
        void dosamples(DCDML_Node Node, DIVE d) throw();
        char *getpersonalinfo(void) throw();
        void builddive(DCDML_Node ParentNode,DIVES &D) throw();
		void adddepth(DCDML_Node Node,int depth,unsigned options=0,int pressure=0,int temp=0,int time=0) throw();
		void addevent(DCDML_Node Node,char *id,int time =0,bool option=0,int heading =-1) throw();
        void DoEvent(TProfileControl &p,DCDML_Node Node) throw();
  		void buildevent(DCDML_Node N,TEvent*E) throw();
        const char * const getrgbmsetting(unsigned s) throw();
        void BuildNodes(DCDMLDocument& Doc,T2NodeStruct &N)  throw();
        void root_generate(DCDMLDocument& Doc,T2NodeStruct &N)  throw();
		void dosettings( DCDML_Node settingsNode) throw();

	public:
		OutputGeneraterT2_D23Root(pDEVICE_DATA p,DIVES *d,int m,int u) : OutputGeneraterT2Base(p,d,m,u)
         {}
		~OutputGeneraterT2_D23Root(){}
		virtual void Generate(DCDMLDocument& Doc)=0;
 };

 class OutputGeneraterT2D2 : public OutputGeneraterT2_D23Root
  {
    private:

	public:
		OutputGeneraterT2D2(pDEVICE_DATA p,DIVES *d,int m,int u) : OutputGeneraterT2_D23Root(p,d,m,u)
         {}
		~OutputGeneraterT2D2(){}
		virtual void Generate(DCDMLDocument& Doc);
  };
 class OutputGeneraterT2D3 : public OutputGeneraterT2_D23Root
  {
    private:
		
	public:
		OutputGeneraterT2D3(pDEVICE_DATA p,DIVES *d,int m,int u) : OutputGeneraterT2_D23Root(p,d,m,u)
         {}
		~OutputGeneraterT2D3(){}
		virtual void Generate(DCDMLDocument& Doc);
  };
 #endif
