#ifndef sAPI_DEVICE_H
   #define sAPI_DEVICE_H

#include <string>
#include <wstdexcept.h>
#include <memory.h>

#include "Headers\sAPI_Comm.h"
#include "Headers\TDCDML.h"
#include "Headers\sAPI_Structures.h"
#include "Headers\sAPI_Stack.h"
#include "Headers\TGenerater.h"
#include "Headers\sAPI_Helper.h"
#include "Headers\DownloadControler.h"
#include "SmartProgress.h"

extern unsigned short MakeWord(unsigned char *c) throw();
extern unsigned char HighByte(unsigned int _A) throw();
extern unsigned char LowByte(unsigned int _A) throw();

class sAPI_Device
 {
     protected:
		DIVES *Dives;
        pDIVE_DATA ADive;
		DEVICE_DATA DeviceInfo;
		void (__stdcall*fProcAction)(int);
		void (__stdcall*fProcStatus)(const wchar_t *);
		int fOutputType;
        const int MemModel;
        int fUnits;
		OutputGenerater *Output;
		friend unsigned short MakeWord(unsigned char *c) throw();
        friend unsigned char HighByte(unsigned int _A) throw();
        friend unsigned char LowByte(unsigned int _A) throw();
        void setunits(unsigned Opt)
         {
         	switch(Opt & ooUnits)
             {
             	case ooUnitsScientific:
                 	fUnits = Units_Scientific;
               	break;
                case ooUnitsMetric:
                	fUnits = Units_Metric;
                    break;
                default:
                	fUnits = Units_Imperial;
             }
         }
        
	  public:
		sAPI_Device(int i) : fProcAction(0),fProcStatus(0),fOutputType(0),
        			Output(0),MemModel(i),fUnits(Units_Imperial)
		 {
			Dives = new DIVES;
			memset(&DeviceInfo,0,sizeof(DeviceInfo));
		 }
		virtual ~sAPI_Device() { delete Dives; if(Output) delete Output; }
		virtual inline int Read(unsigned address,int bytes,BYTE *data) =0;
		virtual inline int Write(unsigned address,int bytes,BYTE *data) =0;
		virtual void Open(int tries,volatile const bool& terminate)=0;
		virtual void InitDownload()=0;
        virtual int Model(void)=0;
		virtual bool DownloadDive(volatile const bool& terminate) =0;
		virtual void DownloadDevice(volatile const bool& terminate)=0;
		virtual void MakeOutput(DCDMLDocument& Doc,unsigned OutputOptions =0)=0;
		TSmartProgress Progress;
		int Units(void){ return fUnits; }

		void __stdcall  SetCallbackRoutines(void (__stdcall *PR)(int),
							 void (__stdcall *SR)(const wchar_t*),
							 void (__stdcall *AR)(int) )
		 {    Progress(PR); fProcStatus = SR; fProcAction = AR;}

		void Action(int val)
		 {	if(fProcAction != NULL) fProcAction(val); return; }
		void Status(const wchar_t *msg)
		 {	if(fProcStatus != NULL) fProcStatus(msg); return; }
		void SetProgress(int val){ Progress[val]; }
		void ClearDiveList(void)
		 {
			if(Dives->size()==0) return;
			for(DIVES::iterator i = Dives->begin(); i != Dives->end(); ++i) delete *i;
			Dives->clear();
		 }
 };

#endif
