
#include <string>
#include <wstdexcept.h>
#include <memory.h>
#include "Headers\sAPI_Comm.h"
#include "Headers\TDCDML.h"
#include "Headers\sAPI_Structures.h"
#include "Headers\sAPI_Stack.h"
#include "SmartProgress.h"

typedef std::list<pDIVE_DATA> DIVES;



class sAPI_Device
 {
     protected:
		DIVES *Dives;
        DIVE_DATA *ADive;
        DEVICE_DATA DeviceInfo;
		void (__stdcall*fProcProgress)(int);
		void (__stdcall*fProcAction)(int);
		void (__stdcall*fProcStatus)(const wchar_t *);
		TSmartProgress Progress;
 		int fOutputType;

	  public:
		sAPI_Device() : fProcProgress(0),fProcAction(0),fProcStatus(0),fOutputType(0)
		 {
			Dives = new DIVES;
			memset(&DeviceInfo,0,sizeof(DeviceInfo));
		 }
		virtual ~sAPI_Device() { delete Dives; }
		virtual void Open(int tries,volatile const bool& terminate)=0;
		virtual void NewDownload()=0;
		virtual bool DownloadDive(volatile const bool& terminate) =0;
		virtual void DownloadDevice(volatile const bool& terminate)=0;
		virtual int Read(unsigned address,int bytes,BYTE *data) =0;
		virtual int Write(unsigned address,int bytes,BYTE *data) =0;
		void __stdcall SetProgressCallbackRoutine(void (__stdcall *routine)(int) )
		{    Progress(routine); } /*fProcProgress = Routine; }*/
		void __stdcall  SetActionCallbackRoutine(void (__stdcall *Routine)(int) )
		 {    fProcAction = Routine;  }
		void __stdcall  SetStatusCallbackRoutine(void (__stdcall *Routine)(const wchar_t*) )
		 {    fProcStatus = Routine; }
		void Action(int val)
		 {	if(fProcAction != NULL) fProcAction(val); return; }
		void Status(const wchar_t *msg)
		 {	if(fProcStatus != NULL) fProcStatus(msg); return; }
		void SetProgress(int val){ Progress[val]; }
		void SetOutputType(int type){ fOutputType = type; }
		virtual void MakeDeviceOutput(DCDMLDocument& Doc)=0;
		virtual void MakeDiveOutput(DCDMLDocument& Doc)=0;
 };

class sAPI_Device_Type1 : public sAPI_Device, public sAPI_CommModel1
 {
	  private:
		bool fIsFirstDive;
		bool fLastDive;
		sAPI_Stack Ss;
		sAPI_Stack *S;
		bool parce_divedata(pDIVE_DATA data);
		void sAPI_Device_Type1::dobyteswap( unsigned short *D);
		void swapbytes(DEVICE_DATA& data);
		void  GetaDivesData(unsigned char cmd,volatile const bool& terminate);
		bool GetFirstDive(pDIVE_DATA Data,volatile const bool& terminate);
		bool GetNextDive(pDIVE_DATA Data,volatile const bool& terminate);

      public:    
          sAPI_Device_Type1(const wchar_t * port,int delay);
		  virtual ~sAPI_Device_Type1();
		  enum sAPI_MemoryModel Model(void) { return mmMEMORYMODEL_1; }
		  void Open(int tries,volatile const bool& terminate);
		  void NewDownload(void);
		  bool DownloadDive(volatile const bool& terminate);
		  void DownloadDevice(volatile const bool& terminate);
		  void MakeDeviceOutput(DCDMLDocument& Doc);
		  void MakeDiveOutput(DCDMLDocument& Doc);
		  int Read(unsigned address,int bytes,BYTE *data){ return sAPI_CommModel1::Read(address,bytes,data);}
		  int Write(unsigned address,int bytes,BYTE *data){return sAPI_CommModel1::Write(address,bytes,data);}
 };

/*
class sAPI_Device_Type2 : public sAPI_Device, public sAPI_CommModel2
 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;
	 public:
		  sAPI_Device_Type2(const wchar_t * port);
		  ~sAPI_Device_Type2();
		  enum sAPI_MemoryModel Model(void) { return mmMEMORYMODEL_2; }
		  void Open(int tries,volatile  const bool& terminate);
		  int __stdcall Read(unsigned address,int bytes,BYTE *data){ return sAPI_CommModel2::Read(address,bytes,data);}
		  int __stdcall Write(unsigned address,int bytes,BYTE *data){return sAPI_CommModel2::Write(address,bytes,data);}
		  void NewDownload(void);
		  bool DownloadDive(volatile const bool& terminate);
		  void DownloadDevice(volatile const bool& terminate);
		  void MakeDeviceOutput(DCDMLDocument& s);
		  void MakeDiveOutput(DCDMLDocument& s);
 };
 */

class sAPI_Device_Type3 : public sAPI_Device, public sAPI_CommModel3
 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;
	 public:
		  sAPI_Device_Type3(const wchar_t * port);
		  virtual ~sAPI_Device_Type3();
		  enum sAPI_MemoryModel Model(void) { return mmMEMORYMODEL_3; }
		  void Open(int tries,volatile const bool& terminate);
		  void NewDownload(void);
		  bool DownloadDive(volatile const bool& terminate);
		  void DownloadDevice(volatile const bool& terminate);
		  void MakeDeviceOutput(DCDMLDocument& s);
		  void MakeDiveOutput(DCDMLDocument& s);
		  int Read(unsigned address,int bytes,BYTE *data){ return sAPI_CommModel3::Read(address,bytes,data);}
		  int Write(unsigned address,int bytes,BYTE *data){return sAPI_CommModel3::Write(address,bytes,data);}
 };

/*
class sAPI_Device_Type2F : public sAPI_Device, public sAPI_CommModel2F

 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;
		//TDiveDownloader *DownloadControl;

	 public:
		  sAPI_Device_Type2F(const wchar_t * port);
		  ~sAPI_Device_Type2F();
		  void Open(int tries,volitile const bool& terminate);
		  int __stdcall Read(unsigned address,int bytes,BYTE *data){ return sAPI_CommModel2F::Read(address,bytes,data);}
		  int __stdcall Write(unsigned address,int bytes,BYTE *data){return sAPI_CommModel2F::Write(address,bytes,data);}
		  void NewDownload(void);
		  bool DownloadDive(bool& terminate);
          void DownloadDevice(bool& terminate);
		  void MakeDeviceOutput(DCDMLDocument& Doc);
		  void MakeDiveOutput(DCDMLDocument& Doc);
 }; */


