

/*

	sAPI_Devices.h -  The Devices Class
			defines the Upper Level Dive Download and Data Tranlation routines.

	Copyright (C) 2008  by James Connell

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef sAPI_DEVICES_H
   #define sAPI_DEVICES_H

#include "TDCDML\TDCDML.h"
#include "Headers\DownloadControler.h"
#include "Headers\Generater.h"

#include "headers\sAPI_Devices_Control.h"

namespace sAPI  {

class Device_Type1 : public Device_Type1_Control
 {
	  private:
		bool fIsFirstDive;
		bool fLastDive;
		sAPI::Stack *S;
		bool parce_divedata(DIVE data) throw(sAPI::BadDiveFormat);
		void dobyteswap( unsigned short *D) throw();
		void swapbytes(DEVICE_DATA& data) throw();
		void GetaDivesData(unsigned char cmd)  throw(sAPI::Error);
		bool GetFirstDive(DIVE Data) throw(sAPI::Error);
		bool GetNextDive(DIVE Data) throw(sAPI::Error);

	  public:
		Device_Type1(const wchar_t * const port, volatile const bool& terminate, int delay);
		~Device_Type1();
		void Open(int tries) throw(sAPI::User_Cancel,sAPI::Device_Not_Present);
		void InitDownload(void) throw(sAPI::Error);
		bool DownloadDive(void) throw(sAPI::Error);
		void DownloadDevice(void) throw(sAPI::Error);
		void MakeOutput(OutputDocument& Doc,unsigned OutputOptions =0) throw(sAPI::Error);
 };


class Device_TypeD : public Device_TypeD_Control
 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;

	 protected:
		TDiveDownloader *DownloadControl;
		PROFILEPTRS P;
		void BuildDeviceData(void);
//		bool isfirstdive(void) const throw() { return fIsFirstDive;}
//		void FirstDiveCompleted(void) throw() { fIsFirstDive = false;}
//		void ResetFirstDive(void) throw() { fIsFirstDive = true;}

	 public:
		Device_TypeD(const wchar_t * const port, volatile const bool& terminate);
		virtual ~Device_TypeD();
		void InitDownload(void) throw(sAPI::Error);
		void Open(int tries) throw(sAPI::User_Cancel,sAPI::Device_Not_Present);
		bool DownloadDive(void) throw(sAPI::Error);
		void DownloadDevice(void) throw(sAPI::Error);
		void MakeOutput(OutputDocument& s,unsigned OutputOptions =0) throw(sAPI::Error);
 };


class Device_TypeC2V2 : public Device_TypeC2V2_Control
 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;

	 protected:
		TDiveDownloader *DownloadControl;
		PROFILEPTRS P;
		void BuildDeviceData(void);
//		bool isfirstdive(void) const throw() { return fIsFirstDive;}
//		void FirstDiveCompleted(void) throw() { fIsFirstDive = false;}
//		void ResetFirstDive(void) throw() { fIsFirstDive = true;}

	 public:
		Device_TypeC2V2(const wchar_t * const port, volatile const bool& terminate);
		virtual ~Device_TypeC2V2();
		void InitDownload(void) throw(sAPI::Error);
		void Open(int tries) throw(sAPI::User_Cancel,sAPI::Device_Not_Present);
		bool DownloadDive(void) throw(sAPI::Error);
		void DownloadDevice(void) throw(sAPI::Error);
		void MakeOutput(OutputDocument& s,unsigned OutputOptions =0) throw(sAPI::Error);
 };

#ifdef USE_FILE_FOR_DEBUG

class Device_TypeDF : public Device, public CommModel2F

 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;
		read_(unsigned address,int bytes,BYTE *data) throw(Error);

	 protected:
		TDiveDownloader *DownloadControl;
		void BuildDeviceData(void);
		bool isfirstdive(void) const  { return fIsFirstDive;}
		void FirstDiveCompleted(void) { fIsFirstDive = false;}
		void ResetFirstDive(void) { fIsFirstDive = true;}
		void gettypedata(void);
		sAPI::Device_Comm *fDevice;
	 public:
		Device_TypeDF(const wchar_t * const port, volatile const bool& terminate=false);
		virtual ~Device_TypeDF();
		void Open(int tries) throw(sAPI::User_Cancel,sAPI::Device_Not_Present);
		void InitDownload(void) throw(sAPI::Error);
		bool DownloadDive(void) throw(sAPI::Error);
		void DownloadDevice(void) throw(sAPI::Error);
		void MakeOutput(OutputDocument& s,unsigned OutputOptions =0) throw(sAPI::Error);
		int Read(unsigned address,int bytes,BYTE *data,bool P=true) throw(sAPI::Error);
		int Write(unsigned address,int bytes,const BYTE *data,bool P=true) throw(sAPI::Error);
		int Command(unsigned send,const unsigned char *cmdstr,unsigned rec,unsigned char* data) throw(sAPI::Error);
 };

class Device_Type3F : public Device, public CommModel3F

 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;
		read_(unsigned address,int bytes,BYTE *data) throw(Error);
		
	 protected:
		TDiveDownloader *DownloadControl;
		void BuildDeviceData(void);
		bool isfirstdive(void) const  { return fIsFirstDive;}
		void FirstDiveCompleted(void) { fIsFirstDive = false;}
		void ResetFirstDive(void) { fIsFirstDive = true;}
		void gettypedata(void);
		sAPI::Device_Comm *fDevice;
	 public:
		Device_Type3F(const wchar_t * const port, volatile const bool& terminate=false);
		virtual ~Device_Type3F();
		void Open(int tries) throw(sAPI::User_Cancel,sAPI::Device_Not_Present);
		void InitDownload(void) throw(sAPI::Error);
		bool DownloadDive(void) throw(sAPI::Error);
		void DownloadDevice(void) throw(sAPI::Error);
		void MakeOutput(OutputDocument& s,unsigned OutputOptions =0) throw(sAPI::Error);
		int Read(unsigned address,int bytes,BYTE *data,bool P=true) throw(sAPI::Error);
		int Write(unsigned address,int bytes,const BYTE *data,bool P=true) throw(sAPI::Error);
		int Command(unsigned send,const unsigned char *cmdstr,unsigned rec,unsigned char* data) throw(sAPI::Error);
 };
 #endif

 }

 #endif

