

/*

	sAPI_DeviceControl.h -  The Device Control Class
			defines the Low Level Interface routines.

	Copyright (C) 2008  by James Connell

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef sAPI_DEVICES_CONTROL_H
   #define sAPI_DEVICES_CONTROL_H

#include "headers\sapi_devicebase.h"

namespace sAPI
{

class Device_Type1_Control : public Device, public CommModel1
 {
	  private:
		void InitDownload(void) {}
		bool DownloadDive(void) { return false; }
		void DownloadDevice(void) {}
		void MakeOutput(OutputDocument& Doc,unsigned OutputOptions =0) throw(Error) {}
		int read_(unsigned address,int bytes,BYTE *data,bool P) throw(Error);
		int write_(unsigned address,int bytes,const BYTE *data,bool P) throw(Error);

	  public:
		Device_Type1_Control(const wchar_t * const port,int delay, volatile const bool& terminate);
		~Device_Type1_Control() {}
		void Open(int tries) throw(User_Cancel,Device_Not_Present);
		int Read(unsigned address,int bytes,BYTE *data,bool P=true) throw(Error);
		int Write(unsigned address,int bytes,const BYTE *data,bool P=true) throw(Error);
		int Command(unsigned send,const unsigned char *cmdstr,unsigned rec,unsigned char* data) throw(Error);
		unsigned char ReadByte(unsigned address,bool P=false) throw(Error);
		unsigned short ReadWord(unsigned address,bool P=false) throw(Error);
		void WriteByte(unsigned address,unsigned char Byte,bool P=false) throw(Error)
		   { Write(address,1,&Byte,P); } 
		void WriteWord(unsigned address,unsigned short Word,bool P=false) throw(Error);
};



class Device_TypeD_Control : public Device, public CommModel2
 {
	 private:
		void InitDownload(void) {}
		bool DownloadDive(void) { return false; }
		void DownloadDevice(void) {}
		void MakeOutput(OutputDocument& s,unsigned OutputOptions =0) {}
		int  read_(unsigned address,int bytes,BYTE *data) throw(Error);
		int  write_(unsigned address,int bytes,const BYTE *data) throw(Error);

	 protected:
		void getheader(void) throw(Error);

	 public:
		Device_TypeD_Control(const wchar_t * const port, volatile const bool& terminate);
		~Device_TypeD_Control(){}
		void Open(int tries) throw(User_Cancel,Device_Not_Present);
		int Read(unsigned address,int bytes,BYTE *data,bool P=true) throw(Error);
		int Write(unsigned address,int bytes,const BYTE *data,bool P=true) throw(Error);
		int Command(unsigned send,const unsigned char *cmdstr,unsigned rec,unsigned char* data) throw(Error);
		unsigned char ReadByte(unsigned address,bool P=false) throw(Error);
		unsigned short ReadWord(unsigned address,bool P=false) throw(Error);
		void WriteByte(unsigned address,unsigned char Byte,bool P=false) throw(Error)
		  { Write(address,1,&Byte,P); }
		void WriteWord(unsigned address,unsigned short Word,bool P=false) throw(Error);

 };


class Device_TypeC2V2_Control : public Device, public CommModel3
 {
	 private:
		bool fIsFirstDive;
		bool fLastDive;
		void InitDownload(void)  {}
		bool DownloadDive(void) { return false; }
		void DownloadDevice(void)  {}
		void MakeOutput(OutputDocument& s,unsigned OutputOptions =0) {}
		int read_(unsigned address,int bytes,BYTE *data) throw(Error);
		int write_(unsigned address,int bytes,const BYTE *data) throw(Error);

	 protected:
		void getheader(void) throw(Error);

	 public:
		Device_TypeC2V2_Control(const wchar_t * const port, volatile const bool& terminate);
		~Device_TypeC2V2_Control() {  }
		void Open(int tries) throw(User_Cancel,Device_Not_Present);
		int Read(unsigned address,int bytes,BYTE *data,bool P=true) throw(Error);
		int Write(unsigned address,int bytes,const BYTE *data,bool P=true) throw(Error);
		int Command(unsigned send,const unsigned char *cmdstr,unsigned rec,unsigned char* data) throw(Error);
		unsigned char ReadByte(unsigned address,bool P=false) throw(Error);
		unsigned short ReadWord(unsigned address,bool P=false) throw(Error);
		void WriteByte(unsigned address,unsigned char Byte,bool P=false) throw(Error)
		  { Write(address,1,&Byte,P); }
		void WriteWord(unsigned address,unsigned short Word,bool P=false) throw(Error);
 };

}


#endif
