

/*
    
    Stack.h -  Class for the Vyper type profile recovery
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef STACK_H
#define STACK_H


#include "Headers/sAPI_Error.h"
#include <boost/utility.hpp>

#define STACKPOINTERVALUE  0x1997

namespace sAPI
{

class Stack : private boost::noncopyable
{
	private:
		size_t fSP;
		const size_t fTop;
		unsigned char *fStack;

	public:

		Stack(size_t n) : fStack(new unsigned char [n+2]),
			fSP(n), fTop(n) 
		 {	 }
		Stack() : fStack(new unsigned char [STACKPOINTERVALUE+2]),
			fSP(STACKPOINTERVALUE), fTop(STACKPOINTERVALUE)
		 {	 }

		~Stack() {	delete[] fStack; }

		unsigned char pop(void)  throw(StackError)
		 {
			if( fSP == fTop ) throw StackEmpty();
			return fStack[++fSP];
		 }

		void push(unsigned char c)  throw(StackError)
		 {
			if( fSP == 0) throw StackError(STACKOVER);
			fStack[fSP--] = c;
		 }
		size_t size(void) const throw() { return fTop - fSP; }
		bool empty(void) const throw() { return fSP == fTop; }
		void clear(void) throw() { fSP = fTop; }
        unsigned char top(void) const throw() { return fStack[fSP]; }
};

}
#endif
