
/*
    
    sAPI_Structures.h -  Divecomputer Internal Memory structure
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/




#ifndef DIVEDATA_COMPDATA_STRUCTS_DEFINED
    #define DIVEDATA_COMPDATA_STRUCTS_DEFINED

#include <list>
#include <vector>
#include <boost\array.hpp>
#include <boost\shared_ptr.hpp>
#include <memory.h>

class sAPI_Device;


struct DownloaderData
 {
	sAPI_Device *D;
	unsigned F;
	unsigned E;
	unsigned C;
 };

struct _DIVE_DATA
 {
	_DIVE_DATA()
	 {
		memset(&Profile,0,(char *)&MemoryModel - (char*)&Profile );
		ProfileLength =0;
		MemoryModel=0;
		Profile = 0;
	 }
	~_DIVE_DATA() {	delete [] Profile; }
	boost::array<int,9> tissue;
	unsigned char *Profile;
	unsigned char SurfaceIntervalMinutes;
	unsigned char SurfaceIntervalHours;
	unsigned char DiveNumber;
	unsigned char SampleInterval;
	unsigned char ConservationSettings;
	unsigned char StartingPressure;
	unsigned char GasMix; /* air mode == 0 */
	unsigned char PPO2;
	unsigned char StartingAirTemp;
	unsigned char Year;
	unsigned char Month;
	unsigned char Day;
	unsigned char Hour;
	unsigned char Minute;
	unsigned char TempAtDepth;
	unsigned char EndingTemp;
	unsigned char EndingPressure;
	unsigned char CNSClock;
	unsigned char GasMix2;
	unsigned char GasMix3;
	unsigned char Second;  // must set this 2 zero if not used!!!!
	unsigned char SurfaceIntervalSeconds;// must set this 2 zero if not used!!!!
	unsigned char TempInterval;
	unsigned char RGBM;
	unsigned char GasModel;
	unsigned short next;
	unsigned short prev;
	unsigned short DiveTime;
	unsigned short MaxDepth;
	unsigned short EventMarker;
	unsigned short PressureIncluded;
	unsigned short SPressure;
	unsigned short EPressure;
	int ProfileLength;
	int MemoryModel;
 };
 
 typedef _DIVE_DATA DIVE_DATA;

 typedef boost::shared_ptr<_DIVE_DATA> DIVE;
 typedef std::list<DIVE> DIVES;

struct __profile_ptrs
 {
	__profile_ptrs() : OldestDive(0),NewestDive(0),DiveCount(0),EmptyDive(0),CurrentDive(0) {}
	~__profile_ptrs() {}
	unsigned short OldestDive;
	unsigned short NewestDive;
	unsigned short EmptyDive;
	unsigned short CurrentDive;
	unsigned short DiveCount;
 };

typedef  __profile_ptrs PROFILEPTRS;


struct _DEVICE_DATA
 {
	_DEVICE_DATA() { memset(&ProfileType,0,((char*)&FirmwareVersionSubMinor+sizeof(int))-(char*)&ProfileType) ;}
    ~_DEVICE_DATA() {}
	int ProfileType;
	unsigned char  UnusedData1[0x1E]; // 00 -> 0x1D
	unsigned short MaxDepth;          // 0x1E,0x1F
	unsigned short TotalDiveTime;     // 0x20,0x21
	unsigned short TotalDives;        // 0x22,0x23
	unsigned char  ComputerModel;     // 0x24
	unsigned char  FirmwareVersionMjr;// 0x25 Is firmware Major Version number
	unsigned char  SerialNumber[4];   // 0x26,0x27,0x28,0x29
	unsigned char  UnknownData1[2];   // 0x2A,0x2B
	unsigned char  PersonalInfo[41];  // Ox2c  -> 0x49
	unsigned char  UnknownData2[7];   // 0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50
	unsigned short RingBufferPTR;     /* 0x51,0x52  not to be written */
	unsigned char  SampleInterval;    // 0x53
	unsigned char  ConservationSettings;  // 0x54
	unsigned char  UnknownData3[7];   // 0x55,0x56,0x57,0x58,0x59,0x5A,0x5B
	unsigned short MaxFreeDiveDepth;  // 0x5C,0x5D
	unsigned short MaxFreeDiveTime;   // 0x5E,0x5F
	unsigned char  UnknownData4;      // 0x60
	unsigned char  ClockStyle;       // 0x61
	unsigned char  Units;             // 0x62
	unsigned char  Mode;  //          // 0x63
	unsigned char  Light; //          // 0x64
	unsigned char  AlarmEnables;      // 0x65
	unsigned short AlarmTime;         // 0x66,0x67
	unsigned short AlarmDepth;        // 0x68,0x69
	unsigned char  UnknownData5[7];   // 0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,0x70
	unsigned char  FirmwareVersionMinor;
	unsigned char  FirmwareVersionSubMinor;
	PROFILEPTRS ProfilePtrs;
 };
 typedef  _DEVICE_DATA DEVICE_DATA;
 typedef  _DEVICE_DATA *pDEVICE_DATA;


#endif
