
/*
    
    DeviceBase.h -  The basic Device Class describes the Dive Computer
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef DEVICEBASE_H
   #define DEVICEBASE_H

#include <string>
#include <memory.h>
#include <boost/utility.hpp>

#include <Trace.h>

#include "Headers\sAPI_Comm.h"
#include "Headers\sAPI_Structures.h"
#include "Headers\sAPI_Stack.h"
#include "Headers\sAPI_Helper.h"
#include "Headers\sAPI_Error.h"
#include <SmartProgress.h>



class DCDMLDocument;
typedef  DCDMLDocument OutputDocument;


class OutputGenerater;

extern unsigned short MakeWord(unsigned char *c) throw();
extern unsigned char HighByte(unsigned int _A) throw();
extern unsigned char LowByte(unsigned int _A) throw();

namespace sAPI
{

class Device : boost::noncopyable
 {
     private:
		int fOutputType;
		const MemoryModel fMemModel;
		const unsigned fMemoryTop;
		int fUnits;
        volatile const bool& fTerminate;
        
     protected:
		DIVES *Dives;
        DIVE ADive;
		DEVICE_DATA DeviceInfo;
		void (*fProcAction)(int);
		void (*fProcStatus)(const wchar_t *);
		OutputGenerater *Output;
		friend unsigned short MakeWord(unsigned char *c) throw();
        friend unsigned char HighByte(unsigned int _A) throw();
        friend unsigned char LowByte(unsigned int _A) throw();
        void setunits(unsigned Opt) throw()
         {
         	switch(Opt & ooUnits)
             {
             	case ooUnitsScientific:
                 	fUnits = Units_Scientific;
               	break;
                case ooUnitsMetric:
                	fUnits = Units_Metric;
                    break;
                default:
                	fUnits = Units_Imperial;
             }
         }
        
	  public:
		Device(MemoryModel m,unsigned j,volatile const bool& t) : fProcAction(0),fProcStatus(0),fOutputType(0),
					Output(0),fMemModel(m),fMemoryTop(j),fUnits(Units_Imperial),fTerminate(t)
		 {
			Dives = new DIVES;
			memset(&DeviceInfo,0,sizeof(DeviceInfo));
		 }
		virtual ~Device() 
		 { 
			 delete Dives; 
	#if !defined(NO_GENERATE)
			 delete Output; 
	#endif
		 }
		virtual int Read(unsigned address,int bytes,BYTE *data,bool P=false) =0;
		virtual int Write(unsigned address,int bytes,const BYTE *data,bool P=false) =0;
		virtual int Command(unsigned send,const unsigned char *cmdstr,unsigned rec,unsigned char* data) =0;
		virtual void Open(int tries)=0;
		virtual void InitDownload()=0;
		virtual bool DownloadDive(void) =0;
	   //	virtual bool DownloadDive(bool direction) =0;
		virtual void DownloadDevice(void)=0;
		virtual void MakeOutput(OutputDocument& Doc,unsigned OutputOptions =0) =0;
		TSmartProgress Progress;
		int Units(void) const throw() { return fUnits; }
		MemoryModel Model(void) const throw() { return fMemModel; }
		unsigned MemoryTop(void) const throw() { return fMemoryTop; }
		const DIVE& GetCurrentDive(void) const throw() { return ADive; }
		const DEVICE_DATA& GetDeviceData(void) const throw() { return DeviceInfo; }
		const DIVES * const GetAllDives(void) const throw() { return Dives;  }
		bool TerminateRaised(void) const throw() { return fTerminate; }
		void  SetCallbackRoutines(void (*PR)(int), void (*SR)(const wchar_t*),void (*AR)(int) ) throw()
		  {  Progress(PR); fProcStatus = SR; fProcAction = AR;}
		void Action(int val) const throw()
		  {	if(fProcAction != NULL) fProcAction(val); return; }
		void Status(const wchar_t *msg) const throw()
		  {	if(fProcStatus != NULL) fProcStatus(msg); return; }
		void SetProgress(int val) throw() { Progress[val]; }
 };

 }
#endif
