#ifndef MICROXMLH
  #define MICROXMLH
// for Borland BCB compatability
  #undef MICROXMLH

#ifndef MICROXMLH_4E9C1E96DCD34eee82EF3519F29CCAC0_H
    #define MICROXMLH_4E9C1E96DCD34eee82EF3519F29CCAC0_H

#include <string>
#include <list>
#include <boost/utility.hpp>

namespace MicroXML
{
          

#ifndef MICROXML_USE_WIDE_EXCEPTION
#include <stdexcept>
#else
#include <wstdexcept.h>
#endif
          
enum FormatOptions { USE_FULLTAGS =1, USE_FORMATTING =2, USE_UTF8 =4, USE_UTF16 =8, USE_NOLOCALE=16};

class Node_class;

typedef Node_class* Node;


struct TAttribs
 {
	std::string Name;
	std::string Value;
	TAttribs( const TAttribs& a ): Name(a.Name),Value(a.Value) {}
	TAttribs( std::string &n,std::string &v ): Name(n),Value(v) {}
	TAttribs(const char *n,std::string &v ): Name(n),Value(v) {}
	TAttribs( std::string &n,const char *v ): Name(n),Value(v) {}
	TAttribs( const char *n,const char *v): Name(n),Value(v) {}
	TAttribs& operator= (const TAttribs& a);
 };

class TAttributes  
 {
	private:
		std::list<TAttribs> Attribs;

	public:
		TAttributes();
		TAttributes(const TAttributes& a);
		~TAttributes(){}
		std::string &operator[](const char* n);
		std::string &operator[](std::string& n);
        std::string &operator[](int n);
		std::string &operator[](size_t n);
		TAttributes& operator= (const TAttributes& a);
	#ifndef MICROXML_USE_WIDE_EXCEPTION
		TAttribs &get(size_t n) throw(std::runtime_error);
	#else
		TAttribs &get(size_t n) throw(w_std::runtime_error);
	#endif
		size_t size(void) const throw();
 };

class Node_class 
 {
    private:
		std::string fName;
		std::string fText;
		std::list<Node_class *> Nodes;
		bool fFormat;
		bool fFullTags;
		bool fComment;
		int fLevel;
		static const std::string comleader;
		static const std::string sp;
		static const std::string opentag;
		static const std::string closetag;
		static const std::string childclosetag;
		static const std::string tagtrailer;
        static const std::string AttHeader;
        static const std::string AttTrailer;
		static const std::string Alpha;
        static const char* const Alias[];
		std::string& format(std::string& t) throw();
		friend class Document;
		
    protected:
		void make(unsigned f,std::string& s) throw();
		void setformat(unsigned x) throw();

	public:
		Node_class();
		Node_class(int l,const char *name);
		Node_class(const Node_class& n);
		~Node_class();
		TAttributes Attributes;
		Node AddChild(const char* const name) throw();
		Node AddChild(std::string& name) throw();
		Node AddComment(const char* const name) throw();
		Node AddComment(std::string &name) throw();
		inline void AddText(const char* const Text) throw() { fText = Text; }
		inline void AddText(std::string& Text) throw() { fText = Text; }
		inline const char* GetText(void) const throw() { return fText.c_str(); }
		inline const std::string& GetTextStr(void) const throw() { return fText; }
		Node_class& operator= (const Node_class& n);
 };

class Document : boost::noncopyable
 {
	private:
		std::string document;
		std::string FileName;
		void * fData;
		Node root;
		unsigned fFormatting;
        bool fDel;
        
	public:
		Document();
		Document(const char *n);
		Document(std::string &n);
		~Document();
		Node Create(const char *name);
		Node Create(std::string &name);
		void Make(void);
		void Make( const std::string &file);
		Document & operator>>(enum FormatOptions f);
		Document & operator<<(enum FormatOptions f);
		void rename(const std::string name) throw();
		void rename(const char* const name)throw();
		const void * data(void) { return fData; }
		//Node find(const char *name);
		//Node find(std::string &name);
 };

#endif

}  // end namespace

#endif
