

#include <vcl.h>
#include <math.h>
#include <trace.h>
#include "Headers\GeneraterT1.h"
#include "Headers\sAPI_Helper.h"

void OutputGeneraterT1D2::Generate(DCDMLDocument& Doc)
 {
	body = Doc.Create("SuuntoDownload");
	Trace(trace::TraceInfo,"Generate::Device(2),Type(1): Begin \'SuuntoDownload\'");

	BuildDeviceData();

	if(fDives->size() > 0)
 	 {
		int q=0;
		for(DIVES::const_iterator  i(fDives->begin()); i != fDives->end(); ++i)
		 {
			Trace(trace::TraceInfo,"Generate::Device(2),Type(1): Dive %u",++q);
			parcedivedata(*i);
		 }
	 }
	else
	 {
		Trace(trace::TraceInfo,"Generate::Device(2),Type(1): No Dives");
		body->AddComment("   No Dives   ");
	 }

	Trace(trace::TraceInfo,"Generate::Make() Building Document");
	Doc.Make();
 }


void OutputGeneraterT1D2::parcedivedata(DIVE Dive)
 {
	DCDML_Node Node;
	DCDML_Node spressNode;
	DCDML_Node stempNode;
	DCDML_Node epressNode;
	DCDML_Node dtempNode;
	DCDML_Node etempNode;
 
	AnsiString a;
	unsigned char * Profileptr((unsigned char *)(&(Dive->SurfaceIntervalMinutes)));
	dives = body->AddChild("DiveData");
	for(int i(0); i<18; ++i) // get dive data struct
	 {
		switch(i)
		 {
			case 4:
				Node = dives->AddChild(DiveDatastrings[i]);
				a = GetPersonalSetting(Profileptr[i]);
				Node->AddText(a.c_str());
				Node = dives->AddChild("AltitudeSetting"); //DiveDatastrings[18]); //"AltitudeSetting"
				a = GetPersonalSetting(Profileptr[i]);
				Node->AddText(a.c_str());
				break;
			case 5: // =Spres
				spressNode = dives->AddChild(DiveDatastrings[i]);
				break;
			case 7:
				Node = dives->AddChild(DiveDatastrings[i]);
				a = 0;
				Node->AddText(a.c_str());
				break;
			case 8:
				stempNode = dives->AddChild(DiveDatastrings[i]);
				break;
			case 14: // =dtemp
				dtempNode = dives->AddChild(DiveDatastrings[i]);
				break;
			case 15: // =Etemp
				etempNode = dives->AddChild(DiveDatastrings[i]);
				break;
			case 16: // =Epress
				epressNode = dives->AddChild(DiveDatastrings[i]);
				break;
			default:
				Node = dives->AddChild(DiveDatastrings[i]);
				a = AnsiString(Profileptr[i]);
				Node->AddText( a.c_str());
		 }
	 }
	DCDML_Node data(dives->AddChild("DiveProfile"));

	buildxmldive(data,Dive);

	a= AnsiString(Dive->SPressure );
	spressNode->AddText(a.c_str());
	a= AnsiString(Dive->EPressure );
	epressNode->AddText(a.c_str());
	a= AnsiString(Dive->StartingAirTemp );
	stempNode->AddText(a.c_str());
	a= AnsiString(Dive->EndingTemp );
	etempNode->AddText(a.c_str());
	a= AnsiString(Dive->TempAtDepth );
	dtempNode->AddText(a.c_str());
 }

AnsiString OutputGeneraterT1D2::buildserialnumber(unsigned char *sn)
 {
	 AnsiString result;
	 for(int i(0); i< 4; ++i)
      {
		int c(sn[i]);
		result += AnsiString(c);
	  }
	 return result;
 }

void OutputGeneraterT1D2::BuildDeviceData(void)
 {
	AnsiString a;
	devicenode = body->AddChild("ComputerSettings");

	a = (int)((fDeviceInfo->MaxDepth / 30.48)+0.5);
	buildsettingstag(devicenode,0,a.c_str());

	a= fDeviceInfo->TotalDiveTime;
	buildsettingstag(devicenode,1,a.c_str());
	a=fDeviceInfo->TotalDives;
	buildsettingstag(devicenode,2,a.c_str());
	a=fDeviceInfo->ComputerModel;
	buildsettingstag(devicenode,3,a.c_str());
	a=fDeviceInfo->MaxFreeDiveDepth;
	buildsettingstag(devicenode,13,a.c_str());
	a=fDeviceInfo->MaxFreeDiveTime;
	buildsettingstag(devicenode,14,a.c_str());
	a=fDeviceInfo->ClockStyle;
	buildsettingstag(devicenode,16,a.c_str());
	a= fDeviceInfo->Units;
	buildsettingstag(devicenode,17,a.c_str());

	a=fDeviceInfo->FirmwareVersionMjr;
	buildsettingstag(devicenode,4,a.c_str());

	a=buildserialnumber(fDeviceInfo->SerialNumber);
	buildsettingstag(devicenode,5,a.c_str());
	a= (char *)fDeviceInfo->PersonalInfo;
	buildsettingstag(devicenode,7,a.c_str());

	a=AnsiString(0);
	buildsettingstag(devicenode,10,a.c_str());
	buildsettingstag(devicenode,11,a.c_str());
	buildsettingstag(devicenode,12,a.c_str());

	/* buildsettingstag(devicenode,24,"50"); */
	buildsettingstag(devicenode,18,a.c_str());
	buildsettingstag(devicenode,19,a.c_str());
	buildsettingstag(devicenode,20,a.c_str());
	buildsettingstag(devicenode,21,a.c_str());
	buildsettingstag(devicenode,22,a.c_str());

}

void OutputGeneraterT1D2::setvariables(TProfileControl& P,int& MD,int &ST,int &ET,int &DT)
 {
	if(P.havetempurature() )
	 {
		if(ST == -99999) ST = P.temp();
		ET =P.temp();
	 }
	if(P.depth() > MD)
	 {
		MD = P.depth();
		DT = ET;
	 }
 }

void OutputGeneraterT1D2::buildxmldive(DCDML_Node Node,DIVE d)
 {
	TProfileControl P(d);
	DCDML_Node X;
	int depth;
	int ET;
	int DT;
	int ft;
	int MD(0);
	int ST(-99999);
	fetchdepthFT(0);
	do
	 {
		if(P.next() == false)  break;
		if(P.isevent())
		 {
		   //	TEvent *E;
			X = Node->AddChild("Datum");
			while((/*E = */P.getevent())!=0)
			 {
			   /*	DCDML_Node N = X->AddChild("event");
				switch(E->fType)
				 {
					case SURFACED:
						 N->AddText("surface");
						break;
					case IS_EVENT:
						buildXMLevent(N,E);
						break;
					case USER_EVENT:
						N->AddText("user");
						break;
				 }
                 */
			 }
			P.eventcomplete();
			if(P.next() == false)  break;
			ft = fetchdepthFT(P.depth());
			setvariables(P,MD,ST,ET,DT);
			X->AddText(AnsiString(ft).c_str());

			continue;
		 }

		else
		 {
			setvariables(P,MD,ST,ET,DT);
			ft = fetchdepthFT(P.depth());

			X = Node->AddChild("Datum");
			X->AddText(AnsiString(ft).c_str());
		 }
	 } while(!P.done());

	d->SPressure = (P.MaxPressure() +5) / 20;
	d->EPressure = (P.MinPressure() +5) / 20;
	
	d->StartingAirTemp =ST;
	d->EndingTemp =ET;
	d->TempAtDepth =DT;
 }


// new version 11/29/07
int OutputGeneraterT1D2::fetchdepthFT(int Datum)
  {
     static int depth;
     int delta;

	 if(Datum == 0)  // Reset()
	  {
		depth = 0;
		return 0;
	  }

	 double newsample(Datum / 30.48) ; // convert cm to ft.

     if(newsample < 1.0 ) //effectivly surfaced!
      {
    	delta = 0 - depth;
        depth = 0;
		Trace(trace::TraceDebug,"Generate::fetchdepth() Current depth less than 1 ft (%3.2f) resetting to 0 ", newsample);
      }
     else
      {
      	delta = ((int)(newsample + 0.5)) - depth;
	    depth += delta;
      }
     Trace(trace::TraceDetail,"Generate::fetchdepth() Feet = %d , Delta = %d",depth,delta);

	 return delta;
  }

void OutputGeneraterT1D2::buildXMLevent(DCDML_Node N,TProfileControl::TEvent* E)
 {
	std::string evnt;
	switch(E->fID)
	 {
		case FAST_ASCENT:
				N->AddText("slow");
				break;
		case DECO:
				N->AddText("deco");
				break;
		case CEILING:
				N->AddText("ceiling");
				break;
	 }
 }

