
/* Generator T2 */

#include <stdexcept>
#include <trace.h>
#include "Headers\GeneraterT2.h"
#include "Headers\sAPI_Helper.h"
#include "Headers\sAPI_Error.h"

using namespace std;

const char * const OutputGeneraterT2Base::event[] = { "slow","violation","user","surface","deco",
						"ceiling","safetystop","safetyceiling","divetime","heading"};

const std::string OutputGeneraterT2Base::DcdmlBase("http://www.DiveComputerDataMarkupLanguage.info/1.0/dcdml");

const char * const OutputGeneraterT2Base::devicestrings[] = {"vendor","Suunto Oy","model","serial","version"};

const char * const OutputGeneraterT2Base::historystrings[] = {"dives","maxdepth","divetime","freedivemaxdepth","freedivetime"};
const char * const OutputGeneraterT2Base::time_str = "time";
const char * const OutputGeneraterT2Base::id_str = "id";
const char * const OutputGeneraterT2Base::units_str = "units";
const char * const OutputGeneraterT2Base::value_str = "value";
const char * const OutputGeneraterT2Base::rgbm_str[] = { "50","100" };

void OutputGeneraterT2Base::basic_generate(DCDMLDocument& Doc,DCDML_Node DCDMLNode) throw(sAPI_CriticalError)
 {
	DCDMLNode->Attributes["version"] = "1.0";
	DCDMLNode->Attributes["tstamp"] = getlocaltime();

	switch(fOutputUnits)
     {
		 case Units_Metric:
			DCDMLNode->Attributes[units_str] = "metric";
            // set unitsconverter to metric
            Units = new MetricUnits(Model());
			break;
		 case Units_Imperial:
			DCDMLNode->Attributes[units_str] = "imperial";
            // set unitsconverter to imperial
            Units = new ImperialUnits(Model());
			break;
		 case Units_Scientific:
			DCDMLNode->Attributes[units_str] = "scientific";
            // set unitsconverter to scientific
            Units = new ScienceUnits(Model());
         default:
			Trace(trace::TraceError,"GenerateT2::SetUnits() Unknown Conversion Units");
            throw sAPI_CriticalError(INVALID_UNITS);
	 }
	Trace(trace::TraceInfo,"GenerateT2::SetUnits( \"%s\" )",DCDMLNode->Attributes[units_str].c_str());
	DCDMLNode->Attributes["xmlns:suunto"] = std::string(DcdmlBase + "-suunto").c_str();;
	Trace(trace::TraceDetail,"GenerateT2::Generater(B) OK");
 }

void OutputGeneraterT2Base::generate_generater(DCDML_Node DCDMLNode) throw()
 {
	// <generator>
	DCDML_Node generatorNode = DCDMLNode->AddChild("generator");
	if(version.Available) generatorNode->Attributes["name"] = version.ProductName.c_str();
	else generatorNode->Attributes["name"] = "Suunto Downloader";
	if(version.Available) generatorNode->Attributes["version"] = version.FileVersion.c_str();
	else generatorNode->Attributes["version"] = "";
	generatorNode->Attributes["vendor"] = "JConn Inv.";
	// </generator>
	Trace(trace::TraceDetail,"GenerateT2::Generater(G) OK");
 }
const char * const OutputGeneraterT2Base::getversioninfo(void) throw()
 {
	sprintf(stringbuffer,"%i.%i.%i",fDeviceInfo->FirmwareVersionMjr,
			fDeviceInfo->FirmwareVersionMinor,fDeviceInfo->FirmwareVersionSubMinor);
	return stringbuffer;
 }

const char * const OutputGeneraterT2Base::getdivemode(void) throw()
 {
	static const char * const modestring[] = { "air","nitrox","gauge","free","" };
	switch(fDeviceInfo->Mode & 0x0F)
     {
		 case 0:
			return modestring[0];
		 case 1:
			return modestring[1];
		 case 2:
			return modestring[2];
		 default:
			if(fDeviceInfo->Mode & 0x80) return modestring[3];
			else
             {
				Trace(trace::TraceInfo,"OutputGeneraterT2Base::getdivemode()  Unknown Dive Mode!!");
             	return modestring[4];
             }
     }
 }

const char * const OutputGeneraterT2Base::UC2str(unsigned char v)  throw()
 {
	return US2str((unsigned short)v);
 }

const char * const OutputGeneraterT2Base::US2str(unsigned short v) throw()
 {
 	sprintf(stringbuffer,"%i",(int)v);
	return stringbuffer;
 }

const char * const OutputGeneraterT2Base::Double2depthstr(double v, unsigned p) throw()
 {
 	switch(p)
     {
     	case Units_Imperial:
        	sprintf(stringbuffer,"%.1f",v); //+0.05
            break;
     	case Units_Metric:
        	sprintf(stringbuffer,"%.0f",v); //+0.5
            break;
     	case Units_Scientific:
        	sprintf(stringbuffer,"%.3f",v); //+0.0005
            break;
        default:
        	sprintf(stringbuffer,"%f",v);
     }
	return stringbuffer;
 }
 
const char * const OutputGeneraterT2Base::Double2pressstr(double v,unsigned p) throw()
 {
 	switch(p)
     {
        default:
     	case Units_Imperial:
        	sprintf(stringbuffer,"%.0f",v); //+0.5
            break;
     	case Units_Metric:
        	sprintf(stringbuffer,"%.1f",v); // + 0.05
            break;
     	case Units_Scientific:
        	sprintf(stringbuffer,"%.2f",v); //+0.005
            break;
     }
	return stringbuffer;
 }

const char * const OutputGeneraterT2Base::getmodelstr(void) const throw()
 {
	 static struct mdlookup M[] =
	  {
		{"Unknown",0},{"Stinger",3},{"Mosquito",4},{"D3",5},{"Viper",0xA},{"Vytec",0xB},{"Cobra",0xC},
		{"Gekko",0xD},{"D9",0xE},{"D6",0xF},{"Viper2",0x10},{"Cobra2",0x11},{"D4",0x12}
	  };
	 const char * result = M[0].Name;
	 for(int i =1; i < 12; i++)
	  {
		if(M[i].ID == fDeviceInfo->ComputerModel)	result = M[i].Name;
	  }
	 return result;
 }

const char * const OutputGeneraterT2Base::getlocaltime(void) throw()
 {
	struct tm *timedate;
	time_t timer = time(NULL);
	timedate = localtime(&timer);
	if( strftime(stringbuffer,200,"%Y-%m-%d T %T",timedate)  == 0 ) stringbuffer[0] = '\0';
	return stringbuffer;
 }

const char * const OutputGeneraterT2Base::getpersonalsetting(unsigned s) throw()
 {
 	sprintf(stringbuffer,"%i",((int)(s & 0xF) / 3));
	return stringbuffer;
 }

const char * const OutputGeneraterT2Base::getaltitudesetting(unsigned s) throw()
 {
 	sprintf(stringbuffer,"%i",((int)(s & 0xF) % 3));
	return stringbuffer;
 }

const char * const OutputGeneraterT2Base::makedivetime(int t) throw()
 {
	int M = t % 60;
	int H = t / 60;
 	sprintf(stringbuffer,"%02u:%02u:00",H,M);
	return stringbuffer;
 }

const char * const OutputGeneraterT2Base::makemaxdepth(int d) throw()
{
    static const char * const z[3] = { "%.1f","%.1f","%.3f" };
    int x;
    double  y=Units->depth(d);
;
    switch(Units->units())
     {
        default:
     	case Units_Imperial:
	        x=0;
        	break;
        case Units_Metric:
	        x=1;
            y/=100;
        	break;
        case Units_Scientific:
	        x=2;
            y/=100;
        	break;
     }
    sprintf(stringbuffer,z[x],y);
	return stringbuffer;
}

void OutputGeneraterT2Base::settimestamp(DCDML_Node Node, DIVE d) throw()
 {
	sprintf(stringbuffer,"%02u-%02u-%02u T %02u:%02u:%02u",(2000+d->Year),d->Month,d->Day,d->Hour,d->Minute,d->Second);
	Node->Attributes[("tstamp")] = stringbuffer;
 }

const char * const OutputGeneraterT2Base::getmode(DIVE d) throw()
 {
	static const char * const m[] = {"air","nitrox","gauge" };
	return m[d->GasModel];
 }

// DONE: need to fix up deciaml points

void OutputGeneraterT2Base::basicadddepth(DCDML_Node Node,int depth, unsigned options,int pressure,int temp,int time) throw()
 {
	DCDML_Node ENode;
	DCDML_Node DepthNode = Node->AddChild(("datum"));
	DepthNode->Attributes[("depth")] = Double2depthstr(Units->depth(depth), Units->units() );
	if(options & PRESSURE) DepthNode->Attributes[("pressure")] = Double2pressstr(Units->pressure(pressure),Units->units());
	if(options & TEMPERATURE) DepthNode->Attributes[("temp")] = US2str(Units->temperature(temp));
	if(options & TIME) DepthNode->Attributes[time_str] = US2str(time);
 }

void OutputGeneraterT2Base::dogasswitch(DCDML_Node Node,unsigned O2,int time) throw()
 {
	//find gas;
    Trace(trace::TraceDetail,"Gas Switch to %d",O2);
	for(MIXLIST::const_iterator i = MixList.begin(); i != MixList.end(); ++i)
     {
		if((*i)->mixO2 == O2)
		 {
			Trace(trace::TraceDetail,"Found %d in list",O2);
			addgasswitch(Node,(*i)->name.c_str(), (time == -1 ? 0: time), (time == -1 ? false : true));
            return;
         }
	 }
	//if not found then define new gasmix
    Trace(trace::TraceDetail,"Adding gas %d to MixList",O2);
    const char * mix = addgastolist(O2,-1,-1);
	addgasswitch(Node,mix, (time == -1 ? 0: time), (time == -1 ? false : true));
 }

void OutputGeneraterT2Base::addgasswitch(DCDML_Node Node, const char *name,int time,bool option) throw()
 {
	DCDML_Node NewNode = Node->AddChild(("gasswitch"));
	NewNode->Attributes[("gas")] = name;
	if(option) NewNode->Attributes[time_str] = AnsiString(time).c_str();
 }

const char * const OutputGeneraterT2Base::addgastolist(unsigned O2,int sp,int ep) throw()
 {
	AnsiString n= AnsiString( AnsiString("Gas")+AnsiString(MixList.size()+1) ) ;
	addgastolist(n.c_str(),O2,sp,ep);
    return n.c_str();
 }

const char * const OutputGeneraterT2Base::addgastolist(const char *name,unsigned O2,int sp,int ep) throw()
 {
	boost::shared_ptr<MixType> mix(new MixType(name,O2,sp,ep));
	MixList.push_back(mix);
    return name;
 }

void OutputGeneraterT2Base::addgasmix(struct MixType &M, DCDML_Node Node) throw()
 {
	static DCDML_Node ParentNode;

 	if(Node) ParentNode = Node;
	Node = ParentNode->AddChild(("gas"));
	Node->Attributes[("name")] = M.name.c_str();
	Node->Attributes[("o2")] = AnsiString(M.mixO2).c_str();
	Trace(trace::TraceDebug,"Adding GasMix: startpressure =%d  endpressure =%d",M.startpress,M.endpress);
	if(M.startpress > 0)
     {
     	Node->Attributes[("startpressure")] = Double2pressstr(Units->pressure(M.startpress),Units->units());
		Node->Attributes[("endpressure")] = Double2pressstr(Units->pressure(M.endpress),Units->units());
     }
 }


void OutputGeneraterT2Base::builduserevent(DCDML_Node Node,TProfileControl::TEvent* E) throw()
 {
	DCDML_Node N = Node->AddChild("event");
	if(short(E->fHeading) < 0)
	 {
		N->Attributes[id_str] = "user";
		N->Attributes[time_str] = AnsiString(E->fTime + E->fSec).c_str();
	 }
	else
	 {
		N->Attributes[id_str] = "heading";
		N->Attributes[value_str] = AnsiString((E->fHeading/2)).c_str();
		N->Attributes[time_str] = AnsiString(E->fSec).c_str();
     }
 }

