

#include <vcl.h>
#include <trace.h>
#include "Headers\GeneraterT2.h"
#include "Headers\sAPI_Helper.h"
#include "Headers\sAPI_Structures.h"


void OutputGeneraterT2_D23Root::root_generate(DCDMLDocument& Doc)  throw()
 {
	dodevice(NodeList.deviceNode);
	dosettings(NodeList.settingsNode);
	dohistory(NodeList.historyNode);
	doprofile(NodeList.profileNode,fDives);   // <--- use list of dives
	Trace(trace::TraceInfo,"Generate::Make() Building Document");
	Doc.Make();
 }

void OutputGeneraterT2_D23Root::dosettings( DCDML_Node settingsNode) throw()
 {
	settingsNode->Attributes[units_str] = fDeviceInfo->Units ?  metric_str : imperial_str;
	DCDML_Node personalinfoNode(settingsNode->AddChild(("suunto:personalinfo")));
	personalinfoNode->Attributes[value_str] = getpersonalinfo();
 }

void OutputGeneraterT2_D23Root::dodevice( DCDML_Node deviceNode) throw()
 {
	deviceNode->Attributes[devicestrings[0]] = devicestrings[1];
	deviceNode->Attributes[devicestrings[2]] = getmodelstr();
	deviceNode->Attributes[devicestrings[3]] = buildserialnumber();
	deviceNode->Attributes[devicestrings[4]] = getversioninfo();

}

char *OutputGeneraterT2_D23Root::buildserialnumber(void) throw()
 {
	unsigned char * data(fDeviceInfo->SerialNumber);
	sprintf(stringbuffer,"%02u%02u%02u%02u",data[0],data[1],data[2],data[3]);
	return stringbuffer;
 }

char *OutputGeneraterT2_D23Root::getpersonalinfo(void) throw()
 {
	strncpy(stringbuffer,fDeviceInfo->PersonalInfo, 40);
	stringbuffer[40] ='\0';
	return stringbuffer;
 }

void OutputGeneraterT2_D23Root::dohistory(DCDML_Node historyNode) throw()
 {
	int i(0);
	historyNode->Attributes[historystrings[i++]] = US2str(fDeviceInfo->TotalDives);
	historyNode->Attributes[historystrings[i++]] = makemaxdepth(fDeviceInfo->MaxDepth);
	historyNode->Attributes[historystrings[i++]] = makedivetime(fDeviceInfo->TotalDiveTime);
	if(fDeviceInfo->MaxFreeDiveDepth > 0)
    	historyNode->Attributes[historystrings[i++]] = US2str(fDeviceInfo->MaxFreeDiveDepth);
	if(fDeviceInfo->MaxFreeDiveTime > 0)
        historyNode->Attributes[historystrings[i++]] = US2str(fDeviceInfo->MaxFreeDiveTime);
 }

void OutputGeneraterT2_D23Root::doprofile(DCDML_Node Node,DIVES *Data) throw()
 {
	bool flag(false);
	DIVES D;
	if(Data->size() == 0)
     {
		 Node->AddComment("  No Dives  ");
		 return;
     }

	DIVES::iterator i(Data->begin());
	 do
      {
		//D.push_front(*i);    // reverse the list
        if( (*i)->DiveNumber ==1)
         {
         	if(flag==false)
             {
             	D.push_back(*i);
				flag = true;
    	        ++i;
        	 	continue;
         	 }
			DCDML_Node GroupNode(Node->AddChild(rep_str));
			builddive(GroupNode,D);
			D.clear();
            flag = false;
        	continue;
         }
        else
         {
         	D.push_back(*i);
            ++i;
         }
	 } while( i != Data->end() );
     
    // do any left overs
    if(D.size()>0)
     {
		DCDML_Node GroupNode(Node->AddChild(rep_str));
        builddive(GroupNode,D);
     }
 }

void OutputGeneraterT2_D23Root::builddive(DCDML_Node ParentNode,DIVES &D) throw()
 {
	const char *c;
	DIVE d;
	DCDML_Node Node;
		#if defined(USE_TRACE)
			static int k(0);
		#endif
	for(DIVES::const_iterator  i(D.begin()); i != D.end(); ++i)
	 {
	 	Trace(trace::TraceInfo,"Generating Dive #%d",k++);
		MixList.clear();
		DCDML_Node DiveNode(ParentNode->AddChild("dive"));
		DCDML_Node SettingsNode(DiveNode->AddChild("settings"));
		DCDML_Node StatsNode(DiveNode->AddChild("statistics"));
		d = *i;
		settimestamp(DiveNode, d);
        // added 12 - 16 - 07
		StatsNode->Attributes["maxdepth"] = Double2depthstr(Units->depth(d->MaxDepth),Units->units());

		StatsNode->Attributes["divetime"] = makedivetime(d->DiveTime);
		sprintf(stringbuffer,"%02u:%02u:%02u",d->SurfaceIntervalHours,
					d->SurfaceIntervalMinutes,d->SurfaceIntervalSeconds);
		DiveNode->Attributes["surfaceinterval"] = stringbuffer;
        DiveNode->Attributes[(id_str)] = US2str(d->DiveNumber);
		SettingsNode->Attributes["computermode"] =getmode(d);
		SettingsNode->Attributes["personalfactor"] = getpersonalsetting(d->ConservationSettings);
		SettingsNode->Attributes["altitudefactor"] = getaltitudesetting(d->ConservationSettings);
		SettingsNode->Attributes["suunto:rgbm"] = getrgbmsetting(d->RGBM);

		 // is this right????
		if (d->CNSClock >= 128)
		 {
			Node = StatsNode->AddChild(("cns"));
			Node->Attributes[("start")] = UC2str(d->CNSClock-128);
		 }
		 else
		  {
			  Node = StatsNode->AddChild(("suunto:otu"));
			  Node->Attributes[value_str] = UC2str(d->CNSClock);
		  }


		// ToDo 4 -c Needs Testing: Does Guage Mode support gas switching???
		switch(d->GasModel)
         {
			 default:
			 case 0: // == air
				addgastolist("Air",21, Units->pressure(d->StartingPressure),Units->pressure(d->EndingPressure));
				break;
			 case 1: //== nitrox
			 case 2: // == gauge
				addgastolist(d->GasMix, Units->pressure(d->StartingPressure),Units->pressure(d->EndingPressure));
				if(d->GasMix2 > 0 /*d->GasMix*/ ) addgastolist(d->GasMix2, 0,0);
				if(d->GasMix3 > 0 /*d->GasMix2*/ ) addgastolist(d->GasMix3, 0,0);
				break;
			 //case 2: // == gauge
				//addgastolist("Gauge",d->GasMix, Units->pressure(d->StartingPressure),Units->pressure(d->EndingPressure));
  		  	//	break;
		 }

		Node = DiveNode->AddChild(("gasdefinitions"));
        // ToDo: does not add the other 2 mixes to output!!!!
		dotissuedata(StatsNode,d);
		dosamples(DiveNode,d);
		// moved here 9/19/07 add routine to change start press/end press in 'd'
		MIXLIST::iterator j(MixList.begin());
		(*j)->startpress = d->SPressure;
		(*j)->endpress = d->EPressure;
		for( ; j != MixList.end(); ++j) addgasmix(*(*j),Node);
	 }
 }

void OutputGeneraterT2_D23Root::dotissuedata(DCDML_Node DiveNode, DIVE d) throw()
 {
	//if(d->tissue->size() == 0) return; // No tissue data
	static const char* TT = "tissue";

	DCDML_Node TissueDataNode(DiveNode->AddChild("tissuedata"));
	for(size_t i(0); i < d->tissue.size(); ++i)
	 {
		DCDML_Node Node(TissueDataNode->AddChild(TT));
		AnsiString T(TT + AnsiString(i+1));
		Node->Attributes[name_str] = T.c_str();
		Node->Attributes[("start")] = US2str(d->tissue[i]);
	}
 }

//TODO: what about "gas switch"???
//TODO: What about 'deco' and deco violations??
void OutputGeneraterT2_D23Root::dosamples(DCDML_Node DiveNode, DIVE d) throw()
 {
	TProfileControl P(d);
	unsigned temp, Options;
	MIXLIST::const_iterator mixlist(MixList.begin());
	DCDML_Node Node(DiveNode->AddChild("samples"));
    Node->Attributes[("interval")] = US2str(d->SampleInterval);
	addgasswitch(Node, (*mixlist)->name.c_str(), 0);
	adddepth(Node,0,TIME,0,0,0);

	do
	 {
		if(P.next() == false)
		 {
			 break;
		 }

		if(P.isevent())
		 {
			DoEvent(P,Node);
			continue;
		 }
		Options=0;
		if(P.havepressure()) Options |= PRESSURE;
		if(P.havetempurature())  Options |= TEMPERATURE;
		adddepth(Node, P.depth(), Options, P.press(), P.temp(), 0);

	 } while(P.done()== false);
	adddepth(Node,0,TIME,0,0,P.time(1));
	d->SPressure = P.MaxPressure();
	d->EPressure = P.MinPressure();
 }

void OutputGeneraterT2_D23Root::adddepth(DCDML_Node Node,int depth, unsigned options,int pressure,int temp,int time) throw()
 {
	basicadddepth(Node,depth,options,pressure,temp,time);
 }
 
void OutputGeneraterT2_D23Root::addevent(DCDML_Node Node,char *id,int time,bool option,int heading) throw()
 {
	DCDML_Node NewNode(Node->AddChild(event_str));
	NewNode->Attributes[id_str] = id;
	if(option & HEADING) NewNode->Attributes[("heading")] = US2str(heading);
	if(option & TIME) NewNode->Attributes[time_str] = US2str(time);
 }

void OutputGeneraterT2_D23Root::DoEvent(TProfileControl &p,DCDML_Node Node) throw()
 {
	TProfileControl::TEvent *E;
	DCDML_Node N;
	while((E = p.getevent())!=0)
	 {
		switch(E->fType)
		 {
			case SURFACED:
				N = Node->AddChild(event_str);
				N->Attributes[id_str] = "surface";
				N->Attributes[time_str] = AnsiString(E->fSec).c_str();
				break;
			case IS_EVENT:
				buildevent(Node,E);
				break;
			case USER_EVENT:
				builduserevent(Node,E);
				break;
			case GASSWITCH_EVENT:
				E->fID = D9GASCHANGE;
				buildevent(Node,E);
				break;
			default:
				Trace(trace::TraceInfo,"Unknown Event Marker: Type = %d",E->fType);

		 }

	 }
	p.eventcomplete();
 }


void OutputGeneraterT2_D23Root::buildevent(DCDML_Node Node,TProfileControl::TEvent* E) throw()
 {
 	static const char * const sAPI_EventString[] =
	{ "unknown", "safety", "slow", "safetyceiling", "deco", "ceiling",
	  "divetime", "depthalarm", "ascend", "gastime", "rgbm","gasswitch"	};
	static const std::string endevent("end_");
	static const std::string suunto("suunto:");
	std::string evnt;
	bool flag(false);

	switch(E->fID)
	 {
		case SAFETY_REQUIRED:
				evnt = suunto + std::string(sAPI_EventString[1]);
				break;
		case FAST_ASCENT:
				evnt = std::string(sAPI_EventString[2]);
				break;
		case SAFETY_MISSED:
				evnt =  suunto + std::string(sAPI_EventString[3]);
				break;
		case DECO:
				evnt = std::string(sAPI_EventString[4]);
				break;
		case CEILING:
				evnt = std::string(sAPI_EventString[5]);
				break;
		case DIVE_TIME:
				evnt = suunto + std::string(sAPI_EventString[6]);
				break;
		case DEPTHALARM:
				evnt = suunto + std::string(sAPI_EventString[7]);
				break;
		case ASCEND:
				evnt = suunto + std::string(sAPI_EventString[8]);
				break;
		case AIR_TIME:
				evnt = suunto + std::string(sAPI_EventString[9]);
				break;
		case RGBM:
				evnt = suunto + std::string(sAPI_EventString[10]);
                flag = true;
				break;
		case D6GASCHANGE:
				dogasswitch(Node,(*(MixList[1])).mixO2,E->fSec + E->fTime);
				return;
		case D9GASCHANGE:
				dogasswitch(Node,E->fGas,E->fSec + E->fTime);
				return;

		// these are "End of Events"
		case SAFETY_REQUIRED+128:
				evnt = suunto + endevent + std::string(sAPI_EventString[1]);
				break;
		case FAST_ASCENT+128:
				evnt = suunto + endevent + std::string(sAPI_EventString[2]);
				break;
		case SAFETY_MISSED+128:
				evnt = suunto + endevent + std::string(sAPI_EventString[3]);
				break;
		case DECO+128:
				evnt = suunto + endevent + std::string(sAPI_EventString[4]);
				break;
		case DIVE_TIME+128:
				evnt = suunto +  endevent + std::string(sAPI_EventString[5]);
				break;
		case CEILING+128:
				evnt = suunto +  endevent + std::string(sAPI_EventString[6]);
				break;
		case DEPTHALARM+128:
				evnt = suunto + endevent + std::string(sAPI_EventString[7]);
				break;
		case ASCEND+128:
				evnt = suunto + endevent + std::string(sAPI_EventString[8]);
				break;
		case AIR_TIME+128:
				evnt = suunto + endevent + std::string(sAPI_EventString[9]);
				break;
		case RGBM+128:
				evnt = suunto + std::string(sAPI_EventString[10]);
				break;
		case D6GASCHANGE+128:
				dogasswitch(Node,(*(MixList[0])).mixO2,E->fSec + E->fTime);
				return;
		default:
				evnt = suunto + std::string(sAPI_EventString[0]);
				flag = true;
				break;
	 }

	DCDML_Node N(Node->AddChild(event_str));
	N->Attributes[id_str] = evnt.c_str();
	if(flag)N->Attributes["suunto:value"] = AnsiString(E->fID).c_str();
	N->Attributes[time_str] = AnsiString(E->fSec + E->fTime).c_str();

 }

const char * const OutputGeneraterT2_D23Root::getrgbmsetting(unsigned s) throw()
 {
	if(s==0x32)	return rgbm_str[0];
    else return rgbm_str[1];
 }


