/*
    
    ProfileControler.cpp -  Profile Interpeter class
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Headers\ProfileControler.h"
#include <trace.h>


TProfileControl::TProfileControl(DIVE d) : fEvent(false), fDone(false),
	fPTR(d->Profile), fEventMarker(d->EventMarker), fTemperatureflag(false),
	fTemperatureInterval(d->TempInterval), fTimeInterval(d->SampleInterval),
	fTime(0), fOffset(0), fProfileLength(d->ProfileLength),fFirstPressure(true),
	fTemperatureCount(d->TempInterval), fCurrentEventMarker(0),
	UsePressure(d->PressureIncluded),fMaxPressure(0),fMinPressure(0),
	fTimeStore(0)
 {
	Event.clear();
	eventitor = Event.begin();
 }

TProfileControl::~TProfileControl()
 {
	Event.clear();
 }

bool TProfileControl::next(void) throw()
 {
	 bool IsTemperatureEvent( fTemperatureInterval == fTemperatureCount ? true:false );

	// if Event then build event data and mark as event occured
	if(fEventMarker == fCurrentEventMarker)  // Is an Event
	 {
		return readevents();
	 }
	// else build depth data

	// increment ftime by sample interval
	fTime = fTimeStore;
	fTimeStore += fTimeInterval;
	// profile done???
	if( need( 4 + (IsTemperatureEvent ? 1 : 0) ))
	 {
      	fDone = true;
		return false;
	 }
	fTemperatureflag = false;

	fDepth = readword();
	if(UsePressure)
	 {
		fPress = readword()/10; 
		if(fFirstPressure)
         {
         	fMaxPressure = fPress;
            fFirstPressure = false;
            Trace(trace::TraceDetail,"TProfileControl::next(): Setting Start Pressure =%u",fMaxPressure);
         }
		fMinPressure = fPress;
		//Trace(TraceDetail,"ProfileControl:: Pressure = %u",fPress);
	 }
	else fPress = 0;

	if(IsTemperatureEvent)
	 {
		 fTemp = (char)readbyte();
		 fTemperatureflag =true;
		 fTemperatureCount =1;
	 }
	else ++fTemperatureCount;
	++fCurrentEventMarker;

	return true;
 }

TProfileControl::TEvent *TProfileControl::getevent(void)  throw()
 {
	if(Event.size()==0) // added 8/17/07
	 {
		fEvent = false;
		return 0;
	 }    // and add
	if(eventitor == Event.end())
	 {
		fEvent = false;
		return 0;
	 }
	return &(*eventitor++);
 }

bool  TProfileControl::need(int x) const throw()
 { // checks if bytes are available in profile
	return ( fOffset + x) > fProfileLength;
 }

bool  TProfileControl::readevents(void)  throw()
 {
	TEvent E;
	Event.clear();

	//bool done(need(1));
	while(!need(1))
	 {
		BYTE type(readbyte());
		fEvent =true;
		switch(type)
		 {
			 case EVENT_MARKER:   // == 1
				if(need(4))
				 {
					eventitor = Event.begin();
					return false;
				 }
				fEventMarker = readword();
				fEventMarker +=readword();
				eventitor = Event.begin();
				return true;

			 case SURFACED:    // ==2
				 if(need(2))
				 {
					eventitor = Event.begin();
					return false;
				 }
				 E.fType = SURFACED;
				 E.fTime = fTime;
				 E.fUN = readbyte();
				 E.fSec = readbyte();
				 Event.push_back(E);
				 break;

			 case IS_EVENT:   // == 3
				 if(need(2))
				 {
					eventitor = Event.begin();
					return false;
				 }
				 E.fType = IS_EVENT;
				 E.fID = readbyte();
				 E.fTime = fTime;
				 E.fSec = readbyte();
				 Event.push_back(E);
				 break;

			 case USER_EVENT:   // == 4
				 if(need(4))
				 {
					eventitor = Event.begin();
					return false;
				 }
				 E.fType = USER_EVENT;
				 E.fTime = fTime;
				 E.fUN = readbyte();
				 E.fSec = readbyte();
				 E.fHeading = readword();
				 Event.push_back(E);
				 break;

			 case GASSWITCH_EVENT:   // == 5
				 if(need(2))
				 {
					eventitor = Event.begin();
					return false;
				 }
				 E.fType = GASSWITCH_EVENT;
				 E.fTime = fTime;
				 E.fGas = readbyte();
				 E.fSec = readbyte();
				 Event.push_back(E);
				 break;

			 default:
				fEvent = false;
				break;   //from true 8/27/07
		 }
	 }
	eventitor = Event.begin();
	return false;
 }

WORD TProfileControl::readword(void)  throw()
 {
	if(need(2)) return 0;

	WORD x(MakeWord(fPTR));
	fPTR += 2;
	fOffset += 2;
	return x;
 }

BYTE TProfileControl::readbyte(void)  throw()
 {
	if(need(1)) return 0;
	++fOffset;
	return *fPTR++;
 }


