#ifndef TDCDMLH
  #define TDCDMLH
// for Borland BCB compatability
  #undef TDCDMLH

#ifndef TDCDML_4E9C1E96DCD34eee82EF3519F29CCAC0_H
    #define TDCDML_4E9C1E96DCD34eee82EF3519F29CCAC0_H

#include <string>
#include <list>
#include <boost/utility.hpp>

#ifndef TDCDML_USE_WIDE_EXCEPTION
#include <stdexcept>
#else
#include <wstdexcept.h>
#endif

#include <AnsiToUtf8.h>
#include <AnsiToUtf16.h>

enum DCDMLFormatOptions { DCDML_USE_FULLTAGS =1, DCDML_USE_FORMATTING =2,DCDML_USE_UTF8 =4,DCDML_USE_UTF16 =8,DCDML_USE_NOLOCALE=16};

class DCDML_Node_class;

typedef DCDML_Node_class* DCDML_Node;


struct TAttribs
 {
	std::string Name;
	std::string Value;
	TAttribs( const TAttribs& a ): Name(a.Name),Value(a.Value) {}
	TAttribs( const std::string &n,std::string& v ): Name(n),Value(v) {}
	TAttribs( const char *n, std::string& v ): Name(n),Value(v) {}
	TAttribs( const std::string& n,const char *v ): Name(n),Value(v) {}
	TAttribs( const char *n,const char *v): Name(n),Value(v) {}
	TAttribs& operator= (const TAttribs& a);
 };

class TAttributes  
 {
	private:
		std::list<TAttribs> Attribs;

	public:
		TAttributes();
		TAttributes(const TAttributes& a);
		~TAttributes(){}
		//std::string &operator[](const char* n);
		std::string &operator[](const std::string& n);
        std::string &operator[](int n);
		std::string &operator[](size_t n);
		TAttributes& operator= (const TAttributes& a);
	#ifndef TDCDML_USE_WIDE_EXCEPTION
		TAttribs &get(size_t n) throw(std::runtime_error);
	#else
		TAttribs &get(size_t n) throw(w_std::runtime_error);
	#endif
		inline size_t size(void) const throw();
 };

class DCDML_Node_class 
 {
    private:
		std::string fName;
		std::string fText;
		std::list<DCDML_Node_class *> Nodes;
		bool fFormat;
		bool fFullTags;
		bool fComment;
		int fLevel;
		static const std::string comleader;
		static const std::string sp;
		static const std::string opentag;
		static const std::string closetag;
		static const std::string childclosetag;
		static const std::string tagtrailer;
        static const std::string AttHeader;
        static const std::string AttTrailer;
		static const std::string Alpha;
        static const char* const Alias[];
		std::string& format(std::string& t) throw();
		friend class DCDMLDocument;
		
    protected:
		void make(unsigned f,std::string& s) throw();
		void setformat(unsigned x) throw();

	public:
		DCDML_Node_class();
		DCDML_Node_class(int l,const std::string& name);
		//DCDML_Node_class(int l,const char *name);
		DCDML_Node_class(const DCDML_Node_class& n);
		~DCDML_Node_class();
		TAttributes Attributes;
		//DCDML_Node AddChild(const char* const name) throw();
		DCDML_Node AddChild(const std::string& name) throw();
		//DCDML_Node AddComment(const char* const name) throw();
		DCDML_Node AddComment(const std::string& name) throw();
		inline void AddText(const char* const Text) throw() { fText = Text; }
		inline void AddText(std::string& Text) throw() { fText = Text; }
		inline const char* GetText(void) const throw() { return fText.c_str(); }
		inline const std::string& GetTextStr(void) const throw() { return fText; }
		DCDML_Node_class& operator= (const DCDML_Node_class& n);
 };

class DCDMLDocument : boost::noncopyable
 {
	private:
		std::string document;
		std::string FileName;
		void * fData;
		DCDML_Node root;
		unsigned fFormatting;
        bool fDel;
        
	public:
		DCDMLDocument();
		//DCDMLDocument(const char *n);
		DCDMLDocument(const std::string& n);
		~DCDMLDocument();
		//DCDML_Node Create(const char *name);
		DCDML_Node Create(const std::string& name);
		void Make(void);
		void Make( const std::string& file);
		DCDMLDocument & operator>>(enum DCDMLFormatOptions f);
		DCDMLDocument & operator<<(enum DCDMLFormatOptions f);
		void rename(const std::string& name) throw();
		//void rename(const char* const name)throw();
		const void * data(void) { return fData; }
 };

#endif
#endif
