

#include "tdcdml\tdcdml.h"

const char* const noencode = "<?xml version=\"1.0\"?>";
const char* const utf8encode = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
const char* const utf16encode = "<?xml version=\"1.0\" encoding=\"UTF-16\"?>";

const std::string DCDML_Node_class::comleader("!--");
const std::string DCDML_Node_class::sp(" ");
const std::string DCDML_Node_class::opentag("<");
const std::string DCDML_Node_class::closetag(" />");
const std::string DCDML_Node_class::childclosetag("</");
const std::string DCDML_Node_class::tagtrailer(" >");
const std::string DCDML_Node_class::AttHeader("=\"");
const std::string DCDML_Node_class::AttTrailer("\"");


#ifndef TDCDML_USE_WIDE_EXCEPTION
	using namespace std;
	#define _tT(x) (x)
	#define tTChar char
#else
	using namespace w_std;
	#define _tT(x) (L##x)
	#define tTChar wchar_t
#endif


const tTChar * const badsubscript = _tT("DCDML::Invalid Attribute Subscript");


DCDMLDocument::DCDMLDocument() : root(0), fData(0),fFormatting(0),fDel(false),FileName("")
 {  }
DCDMLDocument::DCDMLDocument(const char * name) : root(0), fData(0),fFormatting(0),fDel(false),FileName(name)
 {  }
DCDMLDocument::DCDMLDocument(std::string &name) : root(0), fData(0),fFormatting(0),fDel(false),FileName(name)
 {  }

DCDMLDocument::~DCDMLDocument()
 {
	if(root) delete root;
    if(fDel && fData) delete[]fData;
 }

void  DCDMLDocument::Make( void )
 {
	if(fFormatting & DCDML_USE_UTF16)
     {
     	document = utf16encode;
     }
	else if(fFormatting & DCDML_USE_UTF8)
     {
     	document = utf8encode;
     }
	else document = noencode;
	if(fDel && fData) delete[] fData;
	if(fFormatting) document += "\n";
	if(root) root->make(fFormatting,document);
	else document += "";
    
	// ToDo -c Needs Testing: encode document
    // ToDo -c Uses to much memory Add File Option!
	if(fFormatting & DCDML_USE_UTF8)
     {
		 AnsiToUtf8 Utf8(document);
         Utf8.copy(fData);
         fDel =true;
         return;
	 }
	if(fFormatting & DCDML_USE_UTF16)
	 {
     	AnsiToUtf16 Utf16(document);
    	Utf16.copy(fData);
        fDel = true;
        return;
	 } 
	fData = ( void * const )document.c_str();
    fDel = false;
 }

DCDMLDocument & DCDMLDocument::operator>>(enum DCDMLFormatOptions f)
 {
	 if(f & DCDML_USE_FULLTAGS) fFormatting &= ~DCDML_USE_FULLTAGS;
	 if(f & DCDML_USE_FORMATTING)  fFormatting &= ~DCDML_USE_FORMATTING;
	 if(f & DCDML_USE_UTF8) fFormatting &= ~DCDML_USE_UTF8;
	 if(f & DCDML_USE_UTF16)  fFormatting &= ~DCDML_USE_UTF16;
	 if(root) root->setformat(fFormatting);
	 return *this;
 }
 
DCDMLDocument & DCDMLDocument::operator<<(enum DCDMLFormatOptions f)
 {
	 if(f & DCDML_USE_FULLTAGS) fFormatting |= DCDML_USE_FULLTAGS;
	 if(f & DCDML_USE_FORMATTING) fFormatting |= DCDML_USE_FORMATTING;
	 if((f & DCDML_USE_UTF16)&&(f & DCDML_USE_UTF8))
		   throw runtime_error(_tT("TDCDML: Both UTF8 and UTF16 specified"));
	 if(f & DCDML_USE_UTF8)
      {
          fFormatting |= DCDML_USE_UTF8;
          fFormatting &= ~DCDML_USE_UTF16;
      }
	 if(f & DCDML_USE_UTF16)
      {
          fFormatting |= DCDML_USE_UTF16;
          fFormatting &= ~DCDML_USE_UTF8;
      }
	 if(f & DCDML_USE_NOLOCALE)
	  {
          fFormatting &= ~DCDML_USE_UTF8;
          fFormatting &= ~DCDML_USE_UTF16;
      }
	 
	 if(root) root->setformat(fFormatting);
	 return *this;
 }

DCDML_Node DCDMLDocument::Create(const char *name)
 {
	root = new DCDML_Node_class(0,name);
	root->setformat(fFormatting);
	return root;
 }
DCDML_Node DCDMLDocument::Create(std::string &name)
 {
	root = new DCDML_Node_class(0,name.c_str());
	root->setformat(fFormatting);
	return root;
 }

TAttributes::TAttributes(TAttributes& a)
 {
    Attribs  = a.Attribs;
 }

std::string& TAttributes::operator[](const char *n)
 {
	std::string N(n);
    return (*this)[N];
 }

std::string& TAttributes::operator[](std::string& N)
{
 	for(std::list<TAttribs>::iterator i = Attribs.begin(); i != Attribs.end(); ++i)
     {
		if( N == (*i).Name )
		return (*i).Value;
     }
    Attribs.push_back(TAttribs(N,""));
	return Attribs.back().Value;
 }

std::string& TAttributes::operator[](int n)
{ return (*this)[(size_t)n]; }

std::string& TAttributes::operator[](size_t n)
{
	if((n+1) > Attribs.size())  throw runtime_error(badsubscript);
    std::list<TAttribs>::iterator i = Attribs.begin();
	for(size_t z = 0; i != Attribs.end(); ++i,++z) if(z == n) break;
	return (*i).Name;
 }

TAttributes& TAttributes::operator= (TAttributes& a)
 {
    Attribs  = a.Attribs;
	return *this;
 }

TAttribs &TAttributes::get(size_t n) throw(runtime_error)
 {
	if( size() < (size_t)n ) throw runtime_error(badsubscript);

	size_t z =1;
	std::list<TAttribs>::iterator i = Attribs.begin();
	for( ; i!=Attribs.end(); ++i,++z)  if(z == n) break;
	return *i;
 }

size_t TAttributes::size(void) const throw()
 {
    return (int)Attribs.size();
 }

TAttribs& TAttribs::operator= (const TAttribs& a)
 {
	Name = a.Name;
	Value = a.Value;
	return *this;
 }

DCDML_Node_class::DCDML_Node_class() : fLevel(0), fFullTags(false),fFormat(false)
 { }

DCDML_Node_class::DCDML_Node_class(int L,const char *name) : fLevel(L),
		fName(const_cast<char*>(name)),fFullTags(false),fFormat(true)
 { }

DCDML_Node_class::DCDML_Node_class(DCDML_Node n) // copy CTor
 {
    if(n == this) return;
    fName = n->fName;
	Attributes = n->Attributes;
    fFormat = n->fFormat;
	fLevel = n->fLevel;
	fFullTags = n->fFullTags;
    Nodes = n->Nodes;
 }

DCDML_Node_class::~DCDML_Node_class()
 {
    if(Nodes.size() == 0) return;
	for(std::list<DCDML_Node>::iterator i = Nodes.begin(); i!=Nodes.end(); ++i) delete *i;
 }

void DCDML_Node_class::setformat(unsigned x) throw()
 {
   fFullTags = (x & DCDML_USE_FULLTAGS) ? true : false;
   fFormat = (x & DCDML_USE_FORMATTING) ? true : false;
 }

DCDML_Node DCDML_Node_class::AddChild(const char * const name) throw()
 {
	DCDML_Node n = new DCDML_Node_class((this->fLevel)+1,name); // sets up the level
	Nodes.push_back(n);
    return Nodes.back();
 }
 
DCDML_Node DCDML_Node_class::AddChild(std::string &name)  throw()
 {
	return AddChild(name.c_str());
//	DCDML_Node n = new DCDML_Node_class((this->fLevel)+1,name.c_str()); // sets up the level
//	Nodes.push_back(n);
//	return Nodes.back();
 }
 
DCDML_Node DCDML_Node_class::AddComment(const char * const name) throw()
 {
	std::string n(name);
	return AddComment(n);
//	std::string result(comleader);
//	result += std::string(name);
//	return AddChild(result);
 }

DCDML_Node DCDML_Node_class::AddComment(std::string &name) throw()
 {
	std::string result(comleader);
	result += name;
	return AddChild(result.c_str());
 }

void DCDML_Node_class::make(unsigned f,std::string& s) throw()
 {
	setformat(f);
	std::string formatstr("\n");
	// build format string
	if(fFormat) for(int i = fLevel; i > 0; --i) formatstr += "    ";
	else formatstr = "";
	//is comment!!
	if(fName.compare(0,3,comleader)==0)
     {
		 s += formatstr + std::string(opentag + fName ) + std::string("-->");
		 return;
     }

	s += formatstr + std::string(opentag + fName );

	for(size_t i = 1; i <= Attributes.size(); ++i)
     {  // build all attribs
		std::string at =  sp + Attributes.get(i).Name;
		at += std::string(AttHeader + Attributes.get(i).Value + AttTrailer );
		s +=at;
	 }

// then if no children close tag == " />"
    if(Nodes.size() ==0)
     {
		if( fFullTags || (fText.size()!=0)  )
        	s += std::string( tagtrailer + format(fText) + childclosetag + fName + tagtrailer );
        else s += closetag;
		return;
	 }
// else do the children nodes
	s += std::string(tagtrailer + format(fText)); // close lead tag and add Text
	for(std::list<DCDML_Node>::const_iterator i = Nodes.begin(); i != Nodes.end(); ++i)
	 {
		(*i)->make(f,s);
	 }

	// write out the tail string
	if(fFormat) s += formatstr;
	s += std::string( childclosetag + fName + tagtrailer );
 }


const char* const DCDML_Node_class::Alias[] = {"&amp;","&lt;","&gt;","&apos;","&quot;"};
const std::string DCDML_Node_class::Alpha("&<>\'\"");

//Done: need to finish/test format routine to avoid XML errors in text output

std::string& DCDML_Node_class::format(std::string& t) throw()
 {
	size_t z;
	for(size_t i = 0; i != t.size(); ++i)
     {
		if((z=Alpha.find(t[i])) != std::string::npos )
         {
			t.erase(i,1);
			t.insert(i,Alias[z]);
         }
	  }
	return t;
 }
