
/*
    
    CommType1.cpp -  Dive Computer Communication Routines
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "Headers\sAPI_Comm.h"
#include <Trace.h>

extern struct sAPI::COMMOPTIONS CommOption1;

namespace sAPI {

CommModel1::CommModel1(const wchar_t *port, volatile const bool& t,int d) : sAPI::Device_Comm(port,CommOption1,t)
 {
	delay(d);
 }

CommModel1::~CommModel1()
 {
 }

void CommModel1::SetCommState(bool state) throw()
 {
	::ClearCommError(commport(),&ComErrorMask,&ComStat);
	switch(state)
     {
	   case _WRITE:
			SetRTS(true);
            Sleep(16);
			break;

       case _READ:
       		SetRTS(false);
			break;
	 }
	fState = state;
 }

void CommModel1::Talk(int bytes) throw(Error)
 {
     unsigned long sent;
	 unsigned char cksum(getchecksum(CommandBuffer,bytes));
	 ::ClearCommError(commport(),&ComErrorMask,&ComStat);
     CommandBuffer[bytes++] = cksum;
     fEventMask = EV_TXEMPTY;

     SetCommState(_WRITE);
     if(0 == ::WriteFile(commport(),CommandBuffer,bytes,&sent,NULL))
	  {
		SetCommState(_READ);
		throw CommError(ERROR_SEND,L"Talk: Write Serial Port");
	  }

     if( 0 == ::WaitCommEvent(commport(),&fEventMask,0) )
      {
      	SetCommState(_READ);
      	throw CommError(TIMEOUT,L"Talk: Write Serial Timed Out");
      }
	 if(sent !=(unsigned)bytes)
	  {
		SetCommState(_READ);
		throw CommError(ERROR_SEND,L"Talk: Data Send");
	  }
     Sleep(200); // Added 11-21-07
	 Purge();  
	 SetCommState(_READ);

 }

void CommModel1::Listen(int bytes) throw(Error)
 {
	  unsigned long DataLen(0);
	  unsigned long  Len(0);
      unsigned long  B(bytes);

	  if(fState != _READ) SetCommState(_READ);
	  
	  if(0==::ReadFile(commport(),DataBuffer,B,&DataLen,NULL))
       {
			throw CommError(ERROR_RECEIVE,L"Listen: Serial Port Read");
	   }

      if(DataLen == 0)
	   {
		  throw CommError(TIMEOUT,L"Listen: Serial Port Timed Out");
	   }

      if(DataLen != B)
       {
            Len = DataLen;
            if(0==::ReadFile(commport(),&DataBuffer[Len],B-Len,&DataLen,NULL))
             {
				throw CommError(ERROR_RECEIVE,L"Listen: Partial Data Read");
			 }
       }

      if(DataLen+Len != B)
       {
		  throw CommError(TIMEOUT,L"Listen: Partial Read Time Out ");
	   }

	  if(checkpackagechecksum(DataBuffer,bytes-1))
       {
		  throw CommError(BAD_CHECKSUM,L"Listen: Bad Checksum ");
	   }
 }

int CommModel1::GetDataPackage(unsigned char *Buffer,int flag) throw(CommError)
 {
    unsigned long len;
    int count;
    int z(2);

    if(fState !=_READ) SetCommState(_READ);

    // don't care what the first byte is as long as it's 0x08 or 0x09 :0
	if(0==::ReadFile(commport(),Buffer,2,&len,NULL)) // get command and count
     {
		throw CommError(ERROR_RECEIVE);
     }
	switch(len)
     {
        case 0:
			if(flag) return 0;
			throw CommError(TIMEOUT,L"TimeOut");

		case 1:
			if(0==::ReadFile(commport(),&Buffer[1],1,&len,NULL)) // have command, now get count
			 {
				throw CommError(ERROR_RECEIVE);
			 }
			if(len == 0)
			 {
				return 0;
			 }
			break;

		default:
			count = Buffer[1];
			//ToDo: Why is z never reset from init value???? and why is it not hard coded??
			if(0==::ReadFile(commport(),&Buffer[z],count+1,&len,NULL)) // now read in the rest + checksum
			 {
				throw CommError(ERROR_RECEIVE);
			 }
	 }

	if(len != (unsigned long)count+1)
	 {
		throw CommError(TIMEOUT,L"Partial Read");
	 }
	if(checkpackagechecksum(Buffer,count+2))
     {
		throw CommError(BAD_CHECKSUM);
     }

    return count;
 }

 }
