
/*
    
    sAPI_DeviceType1_Control.cpp -  Raw Device control Routines
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "headers\sAPI_Devices_Control.h"

namespace sAPI
{

const int packagesize(32);

Device_Type1_Control::Device_Type1_Control(const wchar_t * const port,
					int delay,volatile const bool& terminate)
     : Device(mmMEMORYMODEL_1,0x1FFF,terminate), CommModel1(port,terminate,delay)
 { }


void Device_Type1_Control::Open(int tries) throw(User_Cancel,Device_Not_Present)
 {
	static const char buffer[] = { 5,0,0x1B,0x1B,0  };
	memcpy(CommandBuffer,buffer,4);
	Progress[tries];
	Progress += tries;
	for(int i(0); i < tries; ++i)
	 {
		if(TerminateRaised() == true) throw(User_Cancel());
		try {
        	Communicate(4,5 + 0x1B);
		 }
		catch(const Error& e) { --Progress; continue; }
		Status(L"Comm Opened Successfully!");
		Progress=0;
		Action(1);
		return;
	 }
	Status(L"Comm Open Failure!");
	Action(0);
	Progress = 0;
	throw( Device_Not_Present(L"Device Type1 Not Found") );
 }

  
int Device_Type1_Control::Command(unsigned send,const unsigned char *cmdstr,
				unsigned rec,unsigned char* data) throw(Error)
 {
    std::memcpy(CommandBuffer,cmdstr,send);
    Communicate(send,rec);
    if(data) std::memcpy(data,&DataBuffer[4],rec);
    return rec;
 }


int Device_Type1_Control::read_(unsigned address,int bytes,BYTE *data,bool P) throw(Error)
 {
     ::Sleep(500); // needs rest between reads!!
	 Purge();

	 /* read command */
  	 CommandBuffer[0] = Command_Read;
  	 CommandBuffer[1] = (unsigned char)((address & 0xff00)>>8); //high address
  	 CommandBuffer[2] = (unsigned char)(address & 0x00ff);
  	 CommandBuffer[3] = (unsigned char) bytes;

     Communicate(4,bytes+5);
     memcpy(data,&DataBuffer[4],bytes);
	 if(P) Progress += bytes;
	 return bytes;
 }


unsigned char Device_Type1_Control::ReadByte(unsigned address,bool P) throw(Error)
 {
	if( static_cast<size_t>(address) > MemoryTop() )
			throw MemoryRangeError();
	unsigned char data;
	read_(address,1,&data,P);
	return data;
 }

unsigned short Device_Type1_Control::ReadWord(unsigned address,bool P) throw(Error)
 {
	if( static_cast<size_t>(address) > (MemoryTop()-1) )
			throw MemoryRangeError();
	unsigned char data[2];
	unsigned short result;
	read_(address,2,data,P);
	result = data[0] <<8;
	result += data[1];
	return result;
 }

int Device_Type1_Control::Read(unsigned address,int bytes,unsigned char *data,bool P) throw(Error)
 {

	if( ( bytes < 1) || ( data == 0) ) throw BadInstruction(BAD_READ_PARAM);
	if( static_cast<size_t>(address+bytes) > (MemoryTop()+1) ) throw MemoryRangeError();

	if(bytes <= packagesize )	return read_(address,bytes,data,P);

	 int r(bytes%packagesize);
	 int z(0);

	 for( int v(bytes/packagesize); v > 0 ;--v)
	  {
		  read_(address+z,packagesize,&data[z],P);
		  z += packagesize;
		  if(TerminateRaised() == true) throw User_Cancel();
      }
	 if(r>0) read_(address+z,r,&data[z],P);
	 return bytes;
 }

int Device_Type1_Control::write_(unsigned address,int bytes,const BYTE *data,bool P) throw(Error)
 {
     ::Sleep(500);
		 /* write data */
  	 CommandBuffer[0] = Command_Write;
  	 CommandBuffer[1] = (unsigned char)((address & 0xff00)>>8);
  	 CommandBuffer[2] = (unsigned char)(address & 0x00ff);
  	 CommandBuffer[3] = (unsigned char)bytes;

	 std::memcpy(&CommandBuffer[4],data,bytes);
	 try {
     	Communicate(bytes+4,5);
	  }
	 catch(const Error& e) { throw; }

     if(DataBuffer[3] != CommandBuffer[3])
      {
		throw CommError(DATA_MISMATCH,L"Write: Data returned did not match sent");
	  }
     Purge();

	 if(P) Progress += bytes;
     return bytes;
 }

void Device_Type1_Control::WriteWord(unsigned address,unsigned short word,bool P) throw(Error)
 {
	unsigned char data[2];
	data[0] = HighByte(word);
	data[1] = LowByte(word);
	Write(address,2,data,P);
 }

 int Device_Type1_Control::Write(unsigned address,int bytes,const BYTE *data,bool P) throw(Error)
 {
	 Purge();
	 if( ( bytes < 1) || ( data == 0) ) throw BadInstruction(BAD_WRITE_PARAM);
	 if( static_cast<size_t>(address+bytes) > (MemoryTop()+1) ) 	throw MemoryRangeError();

 	 ::Sleep(666);// Needs rest between writes!   added 3/8/06
	 CommandBuffer[0] = Command_PreWrite;
  	 CommandBuffer[1] = Command_Command;

	 try {
     	Communicate(2,3);
	  }
	 catch(const Error& e) { throw; }
	 
	 if( bytes <= packagesize )  return write_(address,bytes,data,P);
	 
	 int r(bytes%packagesize);
	 int z(0);
	 
	 for( int v(bytes/packagesize); v > 0 ;--v)
	  {
		  write_(address+z,packagesize,&data[z],P);
		  z += packagesize;
          if(TerminateRaised() == true) throw User_Cancel();          
      }
	 if(r>0) write_(address+z,r,&data[z],P);
     return bytes;
 }

 }
