
#include "headers\sAPI_Devices_Control.h"

sAPI_Device_Type1_Control::sAPI_Device_Type1_Control(const wchar_t * const port,int delay, volatile const bool& terminate)
     : sAPI_Device(mmMEMORYMODEL_1,terminate), sAPI_CommModel1(port,delay)
 { }


void sAPI_Device_Type1_Control::Open(int tries) throw(w_std::runtime_error)
 {
	static const char buffer[] = { 5,0,0x1B,0x1B,0  };
	memcpy(CommandBuffer,buffer,4);
	Progress[tries];
	Progress << tries;
	for(int i=0; i < tries; ++i)
	 {
		if(TerminateRaised() == true)
         {
         	Trace(TraceInfo,"sAPI_Device_Type1::Open() User Cancel Detected");
         	throw(sAPI_User_Cancel());
         }
		try {
        	Communicate(4,5 + 0x1B);
		 }
		catch(w_std::runtime_error& e) { Progress << -1; continue; }
		Status(L"Comm Opened Successfully!");
		Progress=0;
		Action(1);
        return;
	 }
	throw( sAPI_Device_Not_Present(L"Device Type1 Not Found") );
 }

 
int sAPI_Device_Type1_Control::Read(unsigned address,int bytes,BYTE *data) throw(w_std::runtime_error)
 {
	Purge();

    if( (bytes < 1) || (bytes > 32))
     {
		throw sAPI_Error(L"Read: Bytes to Read 1<>32",BAD_READ_PARAM);
     }
     ::Sleep(666); // needs rest between reads!!

	 /* read command */
  	 CommandBuffer[0] = Command_Read;
  	 CommandBuffer[1] = (unsigned char)((address & 0xff00)>>8); //high address
  	 CommandBuffer[2] = (unsigned char)(address & 0x00ff);
  	 CommandBuffer[3] = (unsigned char) bytes;

     Communicate(4,bytes+5);
     memcpy(data,&DataBuffer[4],bytes);
     Progress << bytes;
     return bytes;
 }


int sAPI_Device_Type1_Control::Write(unsigned address,int bytes,const BYTE *data) throw(w_std::runtime_error)
 {
 	 Purge();
	 /* sanity check */
     if( ((bytes < 1) || (bytes > 32)) )
      {
		throw sAPI_Error(L"Write: Bytes to Write 1<>32",BAD_WRITE_PARAM);
	  }
	 ::Sleep(666);// Needs rest between writes!   added 3/8/06
	 /* write command */    /* seems to need this part only for the first write!!!! */
  	 CommandBuffer[0] = Command_PreWrite;
  	 CommandBuffer[1] = Command_Command;

	 try {
     	Communicate(2,3);
	  }
     //catch(){}
	 catch(w_std::runtime_error e) { throw; }

     ::Sleep(666);

		 /* write data */
  	 CommandBuffer[0] = Command_Write;
  	 CommandBuffer[1] = (unsigned char)((address & 0xff00)>>8);
  	 CommandBuffer[2] = (unsigned char)(address & 0x00ff);
  	 CommandBuffer[3] = (unsigned char)bytes;

	 std::memcpy(&CommandBuffer[4],data,bytes);
	 try {
     	Communicate(bytes+4,5);
	  }
	 catch(w_std::runtime_error e) { throw; }

     if(DataBuffer[3] != CommandBuffer[3])
      {
		throw sAPI_CommError(DATA_MISMATCH,L"Write: Data returned did not match sent");
	  }
     Purge();

     Progress << bytes;
     return bytes;
 }

 
int sAPI_Device_Type1_Control::Command(unsigned send,const unsigned char *cmdstr,
                unsigned rec,unsigned char* data) throw(w_std::runtime_error)
 {
    std::memcpy(CommandBuffer,cmdstr,send);
    Communicate(send,rec);
    if(data) std::memcpy(data,&DataBuffer[4],rec);
    return rec;
 }

