
#ifdef USE_FILE_FOR_DEBUG


#include <Trace.h>
#include "headers\sAPI_Devices.h"
#include "headers\sAPI_Helper.h"
#include "headers\GeneraterT1.h"
#include "headers\GeneraterT2.h"
#include "headers\DownloadControler.h"

namespace sAPI {


const int min_packagesize(0x28);
const int max_packagesize(0x78);

Device_TypeDF::Device_TypeDF(const wchar_t * const port,volatile const bool& terminate)
	: Device(mmMEMORYMODEL_2,0x7FFE,terminate), CommModel2F(port), DownloadControl(0)
 {
 	fIsFirstDive = true;
	Dives->clear();   // delete all old dives and reset list

	delete DownloadControl;

	DownloadControl = new TDiveDownloader(this,DeviceInfo.ProfilePtrs);

 }

Device_TypeDF::~Device_TypeDF()
 {
	delete DownloadControl;
 }

void Device_TypeDF::InitDownload(void)  throw(Error)
{
	ResetFirstDive();
	unsigned char buffer[0x28];
	// get profile ptr data
	Read(0x190,0x28,buffer);

	DeviceInfo.ProfilePtrs.DiveCount = MakeWord(&buffer[2]);
	DeviceInfo.ProfilePtrs.OldestDive = MakeWord(&buffer[6]);
	DeviceInfo.ProfilePtrs.NewestDive = MakeWord(&buffer[0]);
	DeviceInfo.ProfilePtrs.EmptyDive = MakeWord(&buffer[4]);
	DeviceInfo.ProfilePtrs.CurrentDive = MakeWord(&buffer[6]);

	delete DownloadControl;

	DownloadControl = new TDiveDownloader(this,DeviceInfo.ProfilePtrs);

	Dives->clear(); // delete all old dives and reset list

	// use header data already read in or get new to calculate size
}

void Device_TypeDF::Open(int tries) throw(User_Cancel,Device_Not_Present)
 {
	CommModel2F::Open(0);
    InitDownload();
 }

void Device_TypeDF::DownloadDevice(void)  throw(Error)
 {
    unsigned char buffer[0x80];
	Progress[0x40 + 0x28 + 0x28 + 0x28 + 100 + 0x78];
	// download SN ,  Units
	Read(0,0x40,buffer);
	Progress += 0x40;
	DeviceInfo.SerialNumber[0] = buffer[0x23];
	DeviceInfo.SerialNumber[1] = buffer[0x24];
	DeviceInfo.SerialNumber[2] = buffer[0x25];
	DeviceInfo.SerialNumber[3] = buffer[0x26];
	DeviceInfo.ComputerModel = buffer[0x28];
	DeviceInfo.Units = (MakeWord(&buffer[0x34]) ? 0:1);

	Read(0x186,0x28,buffer);
	Progress += 0x28;
	DeviceInfo.MaxDepth = MakeWord(&buffer[0]);
	DeviceInfo.TotalDiveTime = MakeWord(&buffer[2]);
	DeviceInfo.TotalDives = MakeWord(&buffer[4]);
	
	Read(0x15E,0x28,buffer);
	Progress += 0x28;
	memcpy(DeviceInfo.PersonalInfo,buffer,41);

	Read(0x190,0x28,buffer);
	Progress += 0x28;

	DeviceInfo.ProfilePtrs.DiveCount = MakeWord(&buffer[2]);
	DeviceInfo.ProfilePtrs.OldestDive = MakeWord(&buffer[6]);
	DeviceInfo.ProfilePtrs.NewestDive = MakeWord(&buffer[0]);
	DeviceInfo.ProfilePtrs.EmptyDive = MakeWord(&buffer[4]);
	DeviceInfo.ProfilePtrs.CurrentDive = MakeWord(&buffer[6]);

	// set total profile size here

	try {
		CommandBuffer[0] =0x0F;
		CommandBuffer[1] =0;
		CommandBuffer[2] =0;
        Communicate(3,8);
	 }
	catch(...)
	 {
		  DeviceInfo.ComputerModel = 0;
		  DeviceInfo.FirmwareVersionMjr = 0;
		  DeviceInfo.FirmwareVersionMinor = 0;
		  DeviceInfo.FirmwareVersionSubMinor = 0;
		  Progress = 100;
		  return;
	 }
	DeviceInfo.ComputerModel = DataBuffer[3];
	DeviceInfo.FirmwareVersionMjr = DataBuffer[4];
	DeviceInfo.FirmwareVersionMinor = DataBuffer[5];
	DeviceInfo.FirmwareVersionSubMinor = DataBuffer[6];

	Read(0,0x78,buffer);
	Progress = 100;
 }

bool Device_TypeDF::DownloadDive(void)  throw(Error)
 {
	if(fIsFirstDive)
	 {
		Progress[DownloadControl->size()];
		fIsFirstDive = false;
	 }

	ADive = DIVE(new DIVE_DATA);
	bool result(DownloadControl->GetDive(ADive));
	if(result) Dives->push_back(ADive);
	return result;
 }

void Device_TypeDF::MakeOutput(DCDMLDocument& Doc,unsigned OutputOptions) throw(Error)
 {
	delete Output;

    setunits(OutputOptions);
	switch(OutputOptions & ooVersion)
	 {
        case 0: // No Type Specified
		case ooVersion1: //OUTPUTTYPE1:
			Trace(trace::TraceInfo,"Device_TypeDF::MakeOutput(): Using Generator Type 1");
			Output = new OutputGeneraterT1D2(&DeviceInfo,Dives,Model());
			break;
		case ooVersion2: //OUTPUTTYPE2:
			Trace(trace::TraceInfo,"Device_TypeDF::MakeOutput(): Using Generator Type 2");
		    Output = new OutputGeneraterT2D2(&DeviceInfo,Dives,Model(),Units());
            break;
		default:
			Trace(trace::TraceError,"Device_TypeDF::MakeOutput(): Requested invalid output Format");
			throw(Error(L"MakeOutput:This format not supported"));
	 }

    Doc << DCDML_USE_UTF8;

	if(OutputOptions & ooUseFullTags) Doc << DCDML_USE_FULLTAGS;
    else Doc >> DCDML_USE_FULLTAGS;

	if(OutputOptions & ooUseFormat) Doc << DCDML_USE_FORMATTING;
    else Doc >> DCDML_USE_FORMATTING;

	Output->Generate(Doc);
	delete Output;
	Output =0;

 }

int Device_TypeDF::Command(unsigned send,const unsigned char *cmdstr,unsigned rec,unsigned char* data) throw(Error)
{ return 0;}

int Device_TypeDF::Read(unsigned address,int bytes,BYTE *data,bool P) throw(Error)
 {
	if( ( bytes < 1) || ( data == 0))
	 {
		Trace(trace::TraceError,"Device_Type2F::Read() Bad Read Command");
		throw BadInstruction(BAD_READ_PARAM);
	 }
	if( static_cast<size_t>(address + bytes) > (MemoryTop()+1) )
	 {
		Trace(trace::TraceError,"Device_Type2F::Read() Memory Overrun Last Address = 0x%X",static_cast<size_t>(address + bytes) );
		throw MemoryRangeError();
	 }

	int totalread(0);

	if( bytes < min_packagesize ) // Small Read
	 {
		unsigned page(address);
		if((address + min_packagesize) > MemoryTop() )  // need to adjust read offset
		 {
			page = MemoryTop() - min_packagesize;
			totalread = read_(page,min_packagesize,0);
			memcpy(data, &DataBuffer[6+(address - page)],bytes);
		 }
		else
		 {
			totalread = read_(page,min_packagesize,0);
			memcpy(data,&DataBuffer[6],bytes);
		 }
		if(P) Progress += totalread;
	 }
	else if( bytes > max_packagesize ) // Large Read
	 {
	 	Trace(trace::TraceDetail,"Device_TypeD::Read() Large Read");
		unsigned add(address);
		const int pages(bytes / max_packagesize);
		for( int i(0); i < pages; ++i)
		 {
			int temp(read_(add,max_packagesize,data));
			if(P) Progress += temp;
			add += temp;
			data += temp;
			totalread += temp;
			if(TerminateRaised() == true)
			 {
				Trace(trace::TraceInfo,"Device_TypeD::Read() User Cancel Detected");
				throw(User_Cancel());
			 }
		 }
		if((bytes - totalread) > 0 )
		 {
			int temp(Read(add,(bytes - totalread),data));
			totalread += temp;
			if(P) Progress += temp;
		 }
	 }
	else
	 {
		totalread = read_( address, bytes, data);
		if(P) Progress += totalread;
	 }
	return totalread;
 }
/*
int Device_TypeDF::Read(unsigned address,int bytes,BYTE *data,bool P) throw(Error)
 {
	if(bytes > 0x78 || bytes < 0x28 )
	 {
		Trace(trace::TraceError,"Device_Type2F::Read 0x%X bytes",bytes);
		throw Error(L"Read: Bytes to Read 0x28<>0x78",BAD_READ_PARAM);
	 }
	if(address + bytes > 0x8000 )
	 {
		Trace(trace::TraceError,"Device_Type2F::Read Address 0x%X",address);
		throw Error(L"Read: Address out of bounds",BAD_READ_PARAM);
	 }
	CommandBuffer[0] =5;
	CommandBuffer[1] =0;
	CommandBuffer[2] =3;
	CommandBuffer[3] = HighByte(address);
	CommandBuffer[4] = LowByte(address);
	CommandBuffer[5] =bytes;
	try {
        Communicate(6,7+bytes);
	}
	catch(...)
     {
		 Trace(trace::TraceError,"Device_Type2F::Read Communication Exception Caught");
		 throw;
     }
	memcpy(data,&DataBuffer[6],bytes);
    return bytes;
 }
*/

int Device_TypeDF::Write(unsigned address,int bytes,const BYTE *data,bool P) throw(Error)
 {

	 return bytes;
 }


#if defined(BORLAND)
	#pragma warn -8057
#endif

int Device_TypeDF::read_(unsigned address,int bytes,BYTE *data) throw(Error)
 {
	CommandBuffer[0] = (unsigned char)Command_Read;
	CommandBuffer[1] = 0;
	CommandBuffer[2] = 3;
	CommandBuffer[3] = HighByte(address);
	CommandBuffer[4] = LowByte(address);
	CommandBuffer[5] = (unsigned char)bytes;
	try {
		Communicate(6,bytes+7);
	 }
	catch(...)
	 {
		Trace(trace::TraceError,"Device_TypeD::Read(): Exception Raised");
		throw;
	 }
	if(data) std::memcpy(data,&DataBuffer[6],bytes);
	return bytes;
 }
#if defined(BORLAND)
	#pragma warn +8057
#endif

} // end namespace

#endif



