
/*
    
    CommType2.cpp -  Dive Computer Communication Routines
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <trace.h>
#include "Headers\sAPI_Comm.h"

extern struct sAPI::COMMOPTIONS CommOption2;

namespace sAPI
{

CommModel2::CommModel2(const wchar_t * const port, volatile const bool& t, size_t tries)
			:  Device_Comm(port,CommOption2,t,tries)
 {  }

CommModel2::~CommModel2()
 {  }

void CommModel2::SetCommState(bool state) throw()
 {
	ClearCommError(commport(),&ComErrorMask,&ComStat);
	switch(state)
     {
        case _READ:
		    SetRTS(true);
			break;
	    case _WRITE:
	        SetRTS(false);
	        Sleep(16);
			break;
	 }
	fState = state;
 }
 
 
void CommModel2::Communicate(int s, int r) throw(Error)
 {
	int t((int)Tries());

    while(1)
     {
        try
         {
			if(terminate()) throw User_Cancel();
			if(s) Talk(s);
			if(terminate()) throw User_Cancel();
			if(r) Listen(r);
		   }
		catch(const User_Cancel& e)
		 {  throw; }
        catch(const CommError& e)
         {
			Trace(trace::TraceError,"DeviceType2_Com::Communicate(): Error  code = %u",e.code());
			if(  t-- > 0 )
             {
				if(terminate()) throw User_Cancel();
				Sleep(100);
				continue;
             }
            throw;
         }
        catch(...) { throw; }
        return;
     }
 }

void CommModel2::Talk(int bytes)  throw(Error)
 {
	 unsigned long sent;
     if(bytes == 0) return;
	 unsigned char cksum(getchecksum(CommandBuffer,bytes));
     if(!commopen()) throw CommError(COMM_NOT_OPEN);

	 ClearCommError(commport(),&ComErrorMask,&ComStat);
	 if(bytes ==0)
	  {
          throw CommError(BAD_WRITE_PARAM,L"Talk: Bytes to Write");
	  }
	 CommandBuffer[bytes++] = cksum;
	 SetCommState(_WRITE);

	 if( 0 == ::WriteFile(commport(),CommandBuffer,bytes,&sent,NULL))
	  {
		SetCommState(_READ);
		throw CommError(ERROR_SEND);
	  }
      // this might need to only read the returned checksum NOT the entire transmision
	 if( 0 == ::ReadFile(commport(),DataBuffer,bytes,&sent,NULL))
	  {
		SetCommState(_READ);
		throw CommError(DEVICE_NOT_PRESENT);
	  }
 }

void CommModel2::Listen(int bytes) throw(Error)
 {
	  if( bytes == 0) return;
	  unsigned long DataLen(0);
	  unsigned long  B(bytes);

	  if(!commopen()) throw CommError(COMM_NOT_OPEN);

	  if(bytes ==0)
	   {
		  throw CommError(BAD_READ_PARAM);
	   }

	  if(fState != _READ) SetCommState(_READ);

	  if( 0 == ::ReadFile(commport(),DataBuffer,B,&DataLen,NULL))
	   {
		SetCommState(_READ);
		if(!commopen()) throw CommError(COMM_NOT_OPEN);
		else
         {
		   Trace(trace::TraceError,"Listen(): Read Timed Out");
           throw CommError(TIMEOUT);
         }
	   }

	  if(DataLen == 0)
	   {
		  throw CommError(TIMEOUT,L"Listen: Serial Port Timed Out");
	   }

	  if(DataLen != B)
	   {
	   		Trace(trace::TraceDebug,"DeviceType2_Com::Listen(): Partial data read %d bytes of %d",DataLen,bytes);
			throw CommError(ERROR_RECEIVE,L"Listen: Partial Data Read");
	   }

	  if(checkpackagechecksum(DataBuffer,bytes-1))
	   {
		  throw CommError(BAD_CHECKSUM);
	   }
 }
}

