
/*
    
    DeviceType2.cpp -  Dive Computer Communication Routines
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "headers\GeneraterT1.h"
#include "headers\GeneraterT2.h"
#include "headers\sAPI_Devices.h"

#include <trace.h>

namespace sAPI  {

Device_TypeD::Device_TypeD(const wchar_t * const port,volatile const bool& terminate)
	: Device_TypeD_Control(port,terminate), DownloadControl(0)
 { }

Device_TypeD::~Device_TypeD()
 {
    delete DownloadControl;
 }


void Device_TypeD::InitDownload(void) throw(Error)
{
	fIsFirstDive = true;
	Dives->clear(); //ClearDiveList(); // delete all old dives and reset list

	delete DownloadControl;

	DownloadControl = new TDiveDownloader(this,DeviceInfo.ProfilePtrs);
}


void Device_TypeD::Open(int tries) throw(User_Cancel,Device_Not_Present)
 {
	Device_TypeD_Control::Open(tries);
	InitDownload();
 }


void Device_TypeD::DownloadDevice(void) throw(Error)
 {
    unsigned char buffer[0x80];
	Progress[0x40 + 0x28 + 0x28 + 0x28 +0x28];
	// download SN ,  Units

	getheader();
	Progress += 0x40;

	Read(0,0x40,buffer);
	Progress += 0x40;
	DeviceInfo.SerialNumber[0] = buffer[0x23];
	DeviceInfo.SerialNumber[1] = buffer[0x24];
	DeviceInfo.SerialNumber[2] = buffer[0x25];
	DeviceInfo.SerialNumber[3] = buffer[0x26];

	DeviceInfo.Units = (MakeWord(&buffer[0x34]) ? 0:1);
	
	if(DeviceInfo.ProfilePtrs.DiveCount != 0)
	 {
		Read(0x186,0x28,buffer);
		DeviceInfo.MaxDepth = MakeWord(&buffer[0]);
		DeviceInfo.TotalDiveTime = MakeWord(&buffer[2]);
		DeviceInfo.TotalDives = MakeWord(&buffer[4]);
		Trace(trace::TraceDetail,"Device: MaxDepth=0x%X,TotalTime=0x%X,TotalDives=0x%X",DeviceInfo.MaxDepth,DeviceInfo.TotalDiveTime,DeviceInfo.TotalDives);
	 }
	else
	 {
		DeviceInfo.MaxDepth = 0;
		DeviceInfo.TotalDiveTime = 0;
		DeviceInfo.TotalDives = 0;
	 }

	Read(0x186,0x28,buffer);
	Progress += 0x28;
	DeviceInfo.MaxDepth = MakeWord(&buffer[0]);
	DeviceInfo.TotalDiveTime = MakeWord(&buffer[2]);
	DeviceInfo.TotalDives = MakeWord(&buffer[4]);

	Read(0x15E,0x28,buffer);
	Progress += 0x28;
	memcpy(DeviceInfo.PersonalInfo,buffer,41);

	Read(0x190,0x28,buffer);
	Progress += 0x28;

	DeviceInfo.ProfilePtrs.DiveCount = MakeWord(&buffer[2]);
	DeviceInfo.ProfilePtrs.OldestDive = MakeWord(&buffer[6]);
	DeviceInfo.ProfilePtrs.NewestDive = MakeWord(&buffer[0]);
	DeviceInfo.ProfilePtrs.EmptyDive = MakeWord(&buffer[4]);
	DeviceInfo.ProfilePtrs.CurrentDive = MakeWord(&buffer[6]);

	Progress = 100;
 }

bool Device_TypeD::DownloadDive(void)  throw(Error)
 {
	if(fIsFirstDive)
	 {
		Progress[DownloadControl->size()];
		fIsFirstDive = false;
	 }

	ADive = DIVE(new DIVE_DATA);
	bool result(DownloadControl->GetDive(ADive));
	if(result) Dives->push_back(ADive);
	return result;
 }

void Device_TypeD::MakeOutput(OutputDocument& Doc,unsigned OutputOptions) throw(Error)
 {

    delete Output;
    Output =0;
    
    setunits(OutputOptions);
	switch(OutputOptions & ooVersion)
	 {
        case 0: // No Type Specified
		case ooVersion1: //OUTPUTTYPE1:
			Trace(trace::TraceInfo,"Device_TypeD::MakeOutput(): Using Generator Type 1");
			Output = new OutputGeneraterT1D2(&DeviceInfo,Dives,Model());
			break;
		case ooVersion2: //OUTPUTTYPE2:
			Trace(trace::TraceInfo,"Device_TypeD::MakeOutput(): Using Generator Type 2");
		 	Output = new OutputGeneraterT2D2(&DeviceInfo,Dives,Model(),Units());
		 	break;
		default:
        	Trace(trace::TraceError,"Device_TypeD::MakeOutput(): Requested invalid output Format");
			throw(Error(L"Output: format not supported"));
	 }

    Doc << DCDML_USE_UTF8;

	if(OutputOptions & ooUseFullTags) Doc << DCDML_USE_FULLTAGS;
    else Doc >> DCDML_USE_FULLTAGS;

	if(OutputOptions & ooUseFormat) Doc << DCDML_USE_FORMATTING;
    else Doc >> DCDML_USE_FORMATTING;

	Output->Generate(Doc);
	delete Output;
	Output =0;
 }
}
