
/*
    
    CommType3.cpp -  Dive Computer Communication Routines
    
    Copyright (C) 2008  by James Connell

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <trace.h>
#include "Headers\sAPI_Comm.h"

extern struct sAPI::COMMOPTIONS CommOption3;

namespace sAPI
{

CommModel3::CommModel3(const wchar_t * const port, volatile const bool& t, size_t tries)
				 : Device_Comm(port,CommOption3,t,tries)
 {  }

CommModel3::~CommModel3()
 {  }

void CommModel3::SetCommState(bool state) throw()
 {
	ClearCommError(commport(),&ComErrorMask,&ComStat);
	switch(state)
	 {
		case _READ:
        	SetRTS(false);
			break;
		case _WRITE:
			SetRTS(true);
        	Sleep(15); // added 11-21-07
			break;
	 }
	fState = state;
 }


void CommModel3::Communicate(int s, int r) throw(Error)
 {
	int t((int)Tries());

    while(1)
     {
        try
         {
			if(terminate()) throw User_Cancel();
			if(s) Talk(s);
			if(r) Listen(r);
		 }
		catch(const User_Cancel& e)
		 {  throw; }
		catch(const CommError& e)
		 {
			Trace(trace::TraceError,"DeviceType3_Com::Communicate(): Errorcode = %u",e.code());
			if(  t-- > 0 )
			 {
				if(terminate()) throw User_Cancel();
				Trace(trace::TraceError,"DeviceType3_Com::Communicate(): Retry");
				Sleep(100);
				continue;
             }
            throw;
         }
        catch(...) { throw; }
        return;
     }
 }


void CommModel3::Talk(int Bytes) throw(Error)
 {
     if(Bytes == 0) return;
	 unsigned long sent;
	 unsigned char cksum(getchecksum(CommandBuffer,Bytes));
	 ::ClearCommError(commport(),&ComErrorMask,&ComStat);
	 CommandBuffer[Bytes++] = cksum;
     fEventMask = EV_TXEMPTY;

	 SetCommState(_WRITE);
	 if(0 == ::WriteFile(commport(),CommandBuffer,Bytes,&sent,NULL))
	  {
		SetCommState(_READ);
		throw CommError(ERROR_SEND,L"Talk: Write Serial Port");
	  }

	 if( 0 == ::WaitCommEvent(commport(),&fEventMask,0) )
      {
      	SetCommState(_READ);
        throw CommError(TIMEOUT,L"Talk: Write Serial Timed Out");
      }
 
	 if(sent != (unsigned)Bytes)
	  {
		SetCommState(_READ);
		throw CommError(ERROR_SEND,L"Talk: Data Send");
	  }
	 Sleep(10);
	 
}


void CommModel3::Listen(int bytes)  throw(Error)
 {
	  if(bytes == 0) return;
	  unsigned long DataLen(0);
	  unsigned long  B(bytes);
	  if(!commopen()) throw CommError(COMM_NOT_OPEN);
	  if(fState != _READ) SetCommState(_READ);

	  if( 0 == ::ReadFile(commport(),DataBuffer,B,&DataLen,NULL))
       {
			Trace(trace::TraceDebug,"DeviceType3_Com::Listen(): Serial Port Error");
			throw CommError(ERROR_RECEIVE,L"Listen: Serial Port Read");
	   }

      if(DataLen == 0)
	   {
			Trace(trace::TraceDebug,"DeviceType3_Com::Listen(): Serial Port Timed Out");
			throw CommError(TIMEOUT,L"Listen: Serial Port Timed Out");
	   }

	  if(DataLen != B)
	   {
			Trace(trace::TraceDebug,"DeviceType3_Com::Listen(): Partial data read %d bytes of %d",DataLen,bytes);
			throw CommError(ERROR_RECEIVE,L"Listen: Partial Data Read");
       }

	  if(checkpackagechecksum(DataBuffer,bytes-1))
       {
       	  Trace(trace::TraceError,"DeviceType3_Com::Listen(): Checksum Error");
		  throw CommError(BAD_CHECKSUM,L"Listen: Bad Checksum ");
	   }
 }
 }
