
#ifndef UnitsConverterH
#define UnitsConverterH

#include <boost/noncopyable.hpp>

#include "Headers\sAPI_Helper.h"
#include "Headers\sAPI_Error.h"

class UnitsConverter : boost::noncopyable
 {
    private:
        static double Feet2CM(int df) throw() { return (df * 30.48); }
		static double CM2Ft(int dcm) throw(){ return  (dcm / 30.48); }
		static int Tf2Tc(int tf) throw(){ return int(tf / 1.8 - 32); }
		static int Tc2Tf(int tc) throw(){ return int(tc * 1.8 + 32); }
		static double PSI2Bar(int p) throw(){ return p / 14.50377377; }
		static double Bar2PSI(int b) throw(){ return b * 14.50377377; }
		static double Bar102PSI(int b) throw(){ return b * 1.450377377; }
		static double _2Bar2PSI(int b) throw(){ return Bar2PSI(2*b); }
		static double _2Bar2Bar(int b) throw(){ return double(b*2); }
		static double Bar102Bar(int b) throw(){ return double(b/10); }
		static double CM2M(int cm) throw(){ return double(cm/100);  }
		static int C2K(int t) throw(){ return t + 273; }
		static int NoConvrt(int v) throw(){ return v; }
		static double NoConvrtD(int v) throw(){ return double(v); }
        
     protected:
		double (*DC)(int d) throw();
		int (*TC)(int t) throw();
        double (*PC)(int p) throw();

     public:
         UnitsConverter(int DeviceType) : TC(&NoConvrt), DC(&NoConvrtD), PC(&NoConvrtD)
          {
              switch(DeviceType)
               {
                   case odDevice1 | Units_Imperial:
                        TC = &Tc2Tf;
                        PC = &_2Bar2PSI;
                        break;
                        
                   case odDevice2 | Units_Imperial:
                   case odDevice3 | Units_Imperial:
                        DC = &CM2Ft;
                        PC = &Bar102PSI;
                        TC = &Tc2Tf;
                        break;

                  case odDevice1 | Units_Metric:
                        DC = &Feet2CM;
                        PC = &_2Bar2Bar;
                        break;
                        
                   case odDevice2 | Units_Metric:
                   case odDevice3 | Units_Metric:
                    	PC = &Bar102Bar;
                        break;

                  case odDevice1 | Units_Scientific:
                    	TC = &C2K;
                        DC = &Feet2CM;
                        PC = &_2Bar2Bar;
                        break;
                        
                   case odDevice2 | Units_Scientific:
                   case odDevice3 | Units_Scientific:
                    	TC = &C2K;
                    	DC = &CM2M;
                    	PC = &Bar102Bar;
                        break;
                        
                   default:
                        throw sAPI::CriticalError(L"UnitsConverter: Unknown Conversion Type ");
               }                  
          }
         virtual ~UnitsConverter(){}   
         virtual double depth(int d) const throw() =0;
         virtual int temperature(int t) const throw() =0;
         virtual int pressure(int p) const throw() =0;
         virtual int units(void) const throw() =0;
};

class ImperialUnits : public UnitsConverter
 {
                
     public:
         ImperialUnits(unsigned DeviceType) : UnitsConverter(DeviceType | Units_Imperial){}
         ~ImperialUnits(){}
         double depth(int d) const throw() { return DC(d); }
         int temperature(int t) const throw() { return int(TC(t)); }
         int pressure(int p)  const throw() { return int(PC(p)); }
         int units(void) const throw() { return Units_Imperial; }
 };

class MetricUnits : public UnitsConverter
 {

     public:
         MetricUnits(unsigned DeviceType) : UnitsConverter(DeviceType | Units_Metric){}
         ~MetricUnits(){}
         double depth(int d) const throw() { return DC(d); }
         int temperature(int t) const throw() { return TC(t); }
         int pressure(int p) const throw() { return int(PC(p)); }
         int units(void) const throw() { return Units_Metric; }
 };

class ScienceUnits : public UnitsConverter
 {

     public:
         ScienceUnits(unsigned DeviceType) : UnitsConverter(DeviceType | Units_Scientific){}
         ~ScienceUnits() {}
         double depth(int d) const throw() { return DC(d)/100; }
         int temperature(int t) const throw() { return int(TC(t)); }
         int pressure(int p) const throw() { return int(PC(p)); }
         int units(void) const throw() { return Units_Scientific; }
};

#endif
